/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: cryptopp.cpp,v 1.61 2005/01/06 18:16:42 jay Exp $
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef PHP_WIN32
#include <iostream>
#include <cmath>
#endif

extern "C" {
#include "php.h"
#include "php_ini.h"
#include "php_globals.h"
#include "ext/standard/info.h"
#include "ext/standard/php_rand.h"
#include "ext/standard/php_lcg.h"

#if PHP_MAJOR_VERSION == 5
#include "zend_exceptions.h"
#endif
}

#include "php_cryptopp.h"


// block cipher algorithms:

#include "j3way.h"
#include "jaes.h"
#include "jblowfish.h"
#include "jcamellia.h"
#include "jcast128.h"
#include "jcast256.h"
#include "jdes.h"
#include "jdes_ede2.h"
#include "jdes_ede3.h"
#include "jdes_xex3.h"
#include "jdiamond2.h"
#include "jdiamond2lite.h"
#include "jgost.h"
#include "jidea.h"
#include "jmars.h"
#include "jrc2.h"
#include "jrc5.h"
#include "jrc6.h"
#include "jsafer.h"
#include "jserpent.h"
#include "jshacal2.h"
#include "jshark.h"
#include "jskipjack.h"
#include "jsquare.h"
#include "jtea.h"
#include "jtwofish.h"


// stream cipher algorithms:

#include "jarc4.h"
#include "jmarc4.h"
#include "jpanamacipher.h"
#include "jseal.h"


// hash algorithms:

#include "jadler32.h"
#include "jcrc32.h"
#include "jhaval.h"
#include "jmd2.h"
#include "jmd4.h"
#include "jmd5.h"
#include "jpanamahash.h"
#include "jripemd160.h"
#include "jsha.h"
#include "jtiger.h"
#include "jwhirlpool.h"

#include "jbasiccipherinfo.h"
#include "jhash.h"
#include "jhmac.h"


// a bunch of macros to make life easier for PHP5 objects...

#include "jobjects.h"



/* True global resources - no need for thread safety here */
static int le_cryptopp_cipher;
static int le_cryptopp_hash;

/* To quickly check if things have been disabled via ini settings... */
static bool cipherEnabled(Cipher TSRMLS_DC);
static bool hashEnabled(Hash TSRMLS_DC);


/* {{{ cryptopp_functions[]
 */
function_entry cryptopp_functions[] = {
	PHP_FE(cryptopp_create_cipher,			NULL)
	PHP_FE(cryptopp_destroy_cipher,			NULL)
	PHP_FE(cryptopp_set_rand_iv,			NULL)
	PHP_FE(cryptopp_set_iv,					NULL)
	PHP_FE(cryptopp_get_iv,					NULL)
	PHP_FE(cryptopp_set_mode,				NULL)
	PHP_FE(cryptopp_get_mode,				NULL)
	PHP_FE(cryptopp_set_padding,			NULL)
	PHP_FE(cryptopp_get_padding,			NULL)
	PHP_FE(cryptopp_set_rng,				NULL)
	PHP_FE(cryptopp_get_rng,				NULL)
	PHP_FE(cryptopp_srand,					NULL)
	PHP_FE(cryptopp_set_plaintext,			NULL)
	PHP_FE(cryptopp_get_plaintext,			NULL)
	PHP_FE(cryptopp_set_ciphertext,			NULL)
	PHP_FE(cryptopp_get_ciphertext,			NULL)
	PHP_FE(cryptopp_set_key,				NULL)
	PHP_FE(cryptopp_get_key,				NULL)
	PHP_FE(cryptopp_set_key_length,			NULL)
	PHP_FE(cryptopp_get_key_length,			NULL)
	PHP_FE(cryptopp_get_default_key_length,	NULL)
	PHP_FE(cryptopp_get_max_key_length,		NULL)
	PHP_FE(cryptopp_get_min_key_length,		NULL)
	PHP_FE(cryptopp_get_mult_key_length,	NULL)
	PHP_FE(cryptopp_get_valid_key_length,	NULL)
	PHP_FE(cryptopp_get_block_size,			NULL)
	PHP_FE(cryptopp_set_rounds,				NULL)
	PHP_FE(cryptopp_get_rounds,				NULL)
	PHP_FE(cryptopp_get_cipher_name,		NULL)
	PHP_FE(cryptopp_get_mode_name,			NULL)
	PHP_FE(cryptopp_get_padding_name,		NULL)
	PHP_FE(cryptopp_get_rng_name,			NULL)
	PHP_FE(cryptopp_get_cipher_type,		NULL)
	PHP_FE(cryptopp_cipher_enabled,			NULL)
	PHP_FE(cryptopp_rng_available,			NULL)
	PHP_FE(cryptopp_encrypt,				NULL)
	PHP_FE(cryptopp_decrypt,				NULL)
	PHP_FE(cryptopp_encrypt_file,			NULL)
	PHP_FE(cryptopp_decrypt_file,			NULL)


	// just some aliases, as 'stream' is now more accurate than 'file'...

	PHP_FALIAS(cryptopp_encrypt_stream, cryptopp_encrypt_file, NULL)
	PHP_FALIAS(cryptopp_decrypt_stream, cryptopp_decrypt_file, NULL)


	// functions specific to the RC2 algorithm...

	PHP_FE(cryptopp_set_effective_key_length,	NULL)
	PHP_FE(cryptopp_get_effective_key_length,	NULL)


	// hash functions...

	PHP_FE(cryptopp_create_hash,		NULL)
	PHP_FE(cryptopp_destroy_hash,		NULL)
	PHP_FE(cryptopp_set_hash_plaintext,	NULL)
	PHP_FE(cryptopp_get_hash_plaintext,	NULL)
	PHP_FE(cryptopp_set_hashtext,		NULL)
	PHP_FE(cryptopp_get_hashtext,		NULL)
	PHP_FE(cryptopp_get_digest_size,	NULL)
	PHP_FE(cryptopp_hash,				NULL)
	PHP_FE(cryptopp_hash_file,			NULL)
	PHP_FE(cryptopp_validate_hash,		NULL)
	PHP_FE(cryptopp_get_hash_name,		NULL)
	PHP_FE(cryptopp_get_hash_type,		NULL)
	PHP_FE(cryptopp_hash_enabled,		NULL)


	// another alias...

	PHP_FALIAS(cryptopp_hash_stream, cryptopp_hash_file, NULL)


	// additional functions for setting HMAC keys...

	PHP_FE(cryptopp_set_hmac_key,			NULL)
	PHP_FE(cryptopp_get_hmac_key,			NULL)
	PHP_FE(cryptopp_set_hmac_key_length,	NULL)
	PHP_FE(cryptopp_get_hmac_key_length,	NULL)

	{NULL, NULL, NULL}
};
/* }}} */


/* {{{ cryptopp_module_entry
 */
zend_module_entry cryptopp_module_entry = {
	STANDARD_MODULE_HEADER,
	"cryptopp",
	cryptopp_functions,
	PHP_MINIT(cryptopp),
	PHP_MSHUTDOWN(cryptopp),
	NULL,
	NULL,
	PHP_MINFO(cryptopp),
	"0.0.15",
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

ZEND_DECLARE_MODULE_GLOBALS(cryptopp)

#ifdef COMPILE_DL_CRYPTOPP
BEGIN_EXTERN_C()
ZEND_GET_MODULE(cryptopp)
END_EXTERN_C()
#endif

/* {{{ INI */
PHP_INI_BEGIN()
	STD_PHP_INI_BOOLEAN("cryptopp.disable_exceptions",			"0", PHP_INI_ALL, OnUpdateBool, ini_disable_exceptions, zend_cryptopp_globals, cryptopp_globals)

	CRYPTOPP_SET_INI(3way)
	CRYPTOPP_SET_INI(aes)
	CRYPTOPP_SET_INI(blowfish)
	CRYPTOPP_SET_INI(camellia)
	CRYPTOPP_SET_INI(cast128)
	CRYPTOPP_SET_INI(cast256)
	CRYPTOPP_SET_INI(des)
	CRYPTOPP_SET_INI(des_ede2)
	CRYPTOPP_SET_INI(des_ede3)
	CRYPTOPP_SET_INI(des_xex3)
	CRYPTOPP_SET_INI(diamond2)
	CRYPTOPP_SET_INI(diamond2_lite)
	CRYPTOPP_SET_INI(gost)
	CRYPTOPP_SET_INI(idea)
	CRYPTOPP_SET_INI(mars)
	CRYPTOPP_SET_INI(rc2)
	CRYPTOPP_SET_INI(rc5)
	CRYPTOPP_SET_INI(rc6)
	CRYPTOPP_SET_INI(safer_k)
	CRYPTOPP_SET_INI(safer_sk)
	CRYPTOPP_SET_INI(shacal2)
	CRYPTOPP_SET_INI(shark)
	CRYPTOPP_SET_INI(serpent)
	CRYPTOPP_SET_INI(skipjack)
	CRYPTOPP_SET_INI(square)
	CRYPTOPP_SET_INI(tea)
	CRYPTOPP_SET_INI(twofish)

	CRYPTOPP_SET_INI(arc4)
	CRYPTOPP_SET_INI(marc4)
	CRYPTOPP_SET_INI(panama_le_cipher)
	CRYPTOPP_SET_INI(panama_be_cipher)
	CRYPTOPP_SET_INI(seal_le)
	CRYPTOPP_SET_INI(seal_be)

	CRYPTOPP_SET_INI(haval)
	CRYPTOPP_SET_INI(haval3)
	CRYPTOPP_SET_INI(haval4)
	CRYPTOPP_SET_INI(haval5)
	CRYPTOPP_SET_INI(md2)
	CRYPTOPP_SET_INI(md4)
	CRYPTOPP_SET_INI(md5)
	CRYPTOPP_SET_INI(panama_le_hash)
	CRYPTOPP_SET_INI(panama_be_hash)
	CRYPTOPP_SET_INI(ripemd128)
	CRYPTOPP_SET_INI(ripemd160)
	CRYPTOPP_SET_INI(ripemd256)
	CRYPTOPP_SET_INI(ripemd320)
	CRYPTOPP_SET_INI(sha1)
	CRYPTOPP_SET_INI(sha256)
	CRYPTOPP_SET_INI(sha384)
	CRYPTOPP_SET_INI(sha512)
	CRYPTOPP_SET_INI(tiger)
	CRYPTOPP_SET_INI(whirlpool)

	CRYPTOPP_SET_INI(md2_hmac)
	CRYPTOPP_SET_INI(md4_hmac)
	CRYPTOPP_SET_INI(md5_hmac)
	CRYPTOPP_SET_INI(ripemd128_hmac)
	CRYPTOPP_SET_INI(ripemd160_hmac)
	CRYPTOPP_SET_INI(ripemd256_hmac)
	CRYPTOPP_SET_INI(ripemd320_hmac)
	CRYPTOPP_SET_INI(sha1_hmac)
	CRYPTOPP_SET_INI(sha256_hmac)
	CRYPTOPP_SET_INI(sha384_hmac)
	CRYPTOPP_SET_INI(sha512_hmac)
	CRYPTOPP_SET_INI(tiger_hmac)
	CRYPTOPP_SET_INI(whirlpool_hmac)

	CRYPTOPP_SET_INI(adler32)
	CRYPTOPP_SET_INI(crc32)
PHP_INI_END()
/* }}} */


/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(cryptopp)
{
	CRYPTOPP_REGISTER_CLASS(cipher, "CryptoppCipher", NULL);
	CRYPTOPP_REGISTER_CLASS(hash, "CryptoppHash", NULL);
	CRYPTOPP_REGISTER_CLASS(exception, "CryptoppException", zend_exception_get_default());


	#ifdef ZTS
	ZEND_INIT_MODULE_GLOBALS(cryptopp, NULL, NULL);
	Z_TYPE(cryptopp_module_entry) = type;
	#endif

	le_cryptopp_cipher = zend_register_list_destructors_ex(destroy_cipher, NULL, "cryptopp cipher", module_number);
	le_cryptopp_hash = zend_register_list_destructors_ex(destroy_hash, NULL, "cryptopp hash", module_number);


	// block ciphers...

	CRYPTOPP_REGISTER_CONST(CIPHER, 3WAY, 			THREEWAY_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, AES, 			AES_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, RIJNDAEL, 		AES_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, BLOWFISH, 		BLOWFISH_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, CAMELLIA, 		CAMELLIA_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, CAST128, 		CAST128_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, CAST256, 		CAST256_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DES, 			DES_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DES_EDE2, 		DES_EDE2_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DES_EDE3, 		DES_EDE3_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DES_XEX3, 		DES_XEX3_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DIAMOND2, 		DIAMOND2_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, DIAMOND2_LITE, 	DIAMOND2_LITE_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, GOST, 			GOST_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, IDEA, 			IDEA_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, MARS, 			MARS_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, RC2, 			RC2_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, RC5, 			RC5_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, RC6, 			RC6_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SAFER_K, 		SAFER_K_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SAFER_SK, 		SAFER_SK_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SERPENT, 		SERPENT_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SHACAL2, 		SHACAL2_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SHARK, 			SHARK_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SKIPJACK, 		SKIPJACK_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, SQUARE, 		SQUARE_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, TEA, 			TEA_CIPHER);
	CRYPTOPP_REGISTER_CONST(CIPHER, TWOFISH, 		TWOFISH_CIPHER);

	// block cipher modes...

	CRYPTOPP_REGISTER_CONST(MODE, ECB, 				ECB_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, CBC, 				CBC_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, CBC_CTS, 			CBC_CTS_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, CFB, 				CFB_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, OFB, 				OFB_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, CTR, 				CTR_MODE);
	CRYPTOPP_REGISTER_CONST(MODE, COUNTER, 			CTR_MODE);

	// block cipher padding settings...

	CRYPTOPP_REGISTER_CONST(PAD, NONE, 				NO_PADDING);
	CRYPTOPP_REGISTER_CONST(PAD, ZEROES, 			ZEROS_PADDING);
	CRYPTOPP_REGISTER_CONST(PAD, PKCS, 				PKCS_PADDING);
	CRYPTOPP_REGISTER_CONST(PAD, ONE_AND_ZEROES, 	ONE_AND_ZEROS_PADDING);
	CRYPTOPP_REGISTER_CONST(PAD, DEFAULT, 			DEFAULT_PADDING);

	// stream ciphers...

	CRYPTOPP_REGISTER_CONST(STREAM, ARC4, 					ARC4_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, MARC4, 					MARC4_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, PANAMA, 				PANAMA_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, PANAMA_LITTLE_ENDIAN, 	PANAMA_LITTLE_ENDIAN_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, PANAMA_BIG_ENDIAN, 		PANAMA_BIG_ENDIAN_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, SEAL, 					SEAL_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, SEAL_LITTLE_ENDIAN, 	SEAL_LITTLE_ENDIAN_STREAM);
	CRYPTOPP_REGISTER_CONST(STREAM, SEAL_BIG_ENDIAN, 		SEAL_BIG_ENDIAN_STREAM);

	// hashes and checksums...

	CRYPTOPP_REGISTER_CONST(CHECKSUM, ADLER32, 				ADLER32_CHECKSUM);
	CRYPTOPP_REGISTER_CONST(CHECKSUM, CRC32, 				CRC32_CHECKSUM);

	CRYPTOPP_REGISTER_CONST(HASH, ADLER32, 					ADLER32_CHECKSUM);
	CRYPTOPP_REGISTER_CONST(HASH, CRC32, 					CRC32_CHECKSUM);
	CRYPTOPP_REGISTER_CONST(HASH, HAVAL, 					HAVAL_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, HAVAL3, 					HAVAL3_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, HAVAL4, 					HAVAL4_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, HAVAL5, 					HAVAL5_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, MD2, 						MD2_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, MD4, 						MD4_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, MD5, 						MD5_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, PANAMA, 					PANAMA_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, PANAMA_LITTLE_ENDIAN, 	PANAMA_LITTLE_ENDIAN_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, PANAMA_BIG_ENDIAN, 		PANAMA_BIG_ENDIAN_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, RIPEMD128, 				RIPEMD128_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, RIPEMD160, 				RIPEMD160_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, RIPEMD256, 				RIPEMD256_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, RIPEMD320, 				RIPEMD320_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, SHA, 						SHA1_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, SHA1, 					SHA1_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, SHA256, 					SHA256_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, SHA384, 					SHA384_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, SHA512, 					SHA512_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, TIGER, 					TIGER_HASH);
	CRYPTOPP_REGISTER_CONST(HASH, WHIRLPOOL, 				WHIRLPOOL_HASH);

	// HMACs...

	CRYPTOPP_REGISTER_CONST(HMAC, MD2, 						MD2_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, MD4, 						MD4_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, MD5, 						MD5_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, RIPEMD128, 				RIPEMD128_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, RIPEMD160, 				RIPEMD160_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, RIPEMD256, 				RIPEMD256_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, RIPEMD320, 				RIPEMD320_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, SHA, 						SHA1_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, SHA1, 					SHA1_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, SHA256, 					SHA256_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, SHA384, 					SHA384_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, SHA512, 					SHA512_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, TIGER, 					TIGER_HMAC);
	CRYPTOPP_REGISTER_CONST(HMAC, WHIRLPOOL, 				WHIRLPOOL_HMAC);

	// random number generators...

	CRYPTOPP_REGISTER_CONST(RNG, NONBLOCKING, 				NONBLOCKING_RNG);
	CRYPTOPP_REGISTER_CONST(RNG, BLOCKING, 					BLOCKING_RNG);
	CRYPTOPP_REGISTER_CONST(RNG, RAND, 						RAND_RNG);
	CRYPTOPP_REGISTER_CONST(RNG, DEFAULT, 					DEFAULT_RNG);

	REGISTER_INI_ENTRIES();

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(cryptopp)
{
	UNREGISTER_INI_ENTRIES();

	return SUCCESS;
}
/* }}} */

#define CRYPTOPP_SHOW_CIPHER_ENABLED(name, constant) \
	php_info_print_table_row(2, name, (cipherEnabled(constant TSRMLS_CC) ? "enabled" : "disabled"));
#define CRYPTOPP_SHOW_HASH_ENABLED(name, constant) \
	php_info_print_table_row(2, name, (hashEnabled(constant TSRMLS_CC) ? "enabled" : "disabled"));

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(cryptopp)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "cryptopp support", "enabled");
	php_info_print_table_header(2, "version", cryptopp_module_entry.version);
	php_info_print_table_colspan_header(2, (char*) string(string("compiled as ") + string(
	#ifdef COMPILE_DL_CRYPTOPP
	"shared"
	#else
	"static"
	#endif
	) + string(" module")).c_str());
	php_info_print_table_colspan_header(2, (char *) string(string("object-oriented interface ") + string(
	#ifdef USE_PHP5_OBJECTS
	"enabled (ZE2 available)"
	#else
	"disabled (ZE2 unavailable)"
	#endif
	)).c_str());
	php_info_print_table_colspan_header(2, "Block Cipher Algorithms");
	CRYPTOPP_SHOW_CIPHER_ENABLED("3-Way",THREEWAY_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("AES (Rijndael)", AES_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Blowfish", BLOWFISH_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Camellia", CAMELLIA_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("CAST-128", CAST128_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("CAST-256", CAST256_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("DES", DES_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("DES-EDE2 (Triple DES)", DES_EDE2_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("DES-EDE3 (Triple DES)",  DES_EDE3_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("DES-XEX3 (DESX)", DES_XEX3_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Diamond2", DIAMOND2_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Diamond2 Lite",  DIAMOND2_LITE_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("GOST", GOST_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("IDEA", IDEA_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("MARS", MARS_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("RC2", RC2_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("RC5", RC5_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("RC6", RC6_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SAFER-K", SAFER_K_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SAFER-SK", SAFER_SK_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SHARK-E", SHARK_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SHACAL2", SHACAL2_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Serpent", SERPENT_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SKIPJACK", SKIPJACK_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Square", SQUARE_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("TEA", TEA_CIPHER);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Twofish", TWOFISH_CIPHER);

	php_info_print_table_colspan_header(2, "Block Modes");
	php_info_print_table_row(2, "Electronic Code Book (ECB)",			"available");
	php_info_print_table_row(2, "Cipher Block Chaining (CBC)",			"available");
	php_info_print_table_row(2, "CBC with Ciphertext Stealing (CTS)",	"available");
	php_info_print_table_row(2, "Cipher Feedback (CFB)",				"available");
	php_info_print_table_row(2, "Output Feedback (OFB)",				"available");
	php_info_print_table_row(2, "Counter (CTR)",						"available");

	php_info_print_table_colspan_header(2, "Block Mode Padding");
	php_info_print_table_row(2, "None",									"available");
	php_info_print_table_row(2, "Zeroes",								"available");
	php_info_print_table_row(2, "PKCS",									"available");
	php_info_print_table_row(2, "One and Zeroes",						"available");
	php_info_print_table_row(2, "Default (PKCS for ECB and CBC, None for other modes)",	"available");

	php_info_print_table_colspan_header(2, "Stream Cipher Algorithms");
	CRYPTOPP_SHOW_CIPHER_ENABLED("ARC4", ARC4_STREAM);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Modified ARC4 (MARC4)", MARC4_STREAM);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Panama-LE Cipher", PANAMA_LITTLE_ENDIAN_STREAM);
	CRYPTOPP_SHOW_CIPHER_ENABLED("Panama-BE Cipher", PANAMA_BIG_ENDIAN_STREAM);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SEAL-3.0-BE", SEAL_BIG_ENDIAN_STREAM);
	CRYPTOPP_SHOW_CIPHER_ENABLED("SEAL-3.0-LE", SEAL_LITTLE_ENDIAN_STREAM);

	php_info_print_table_colspan_header(2, "Hash Algorithms");
	CRYPTOPP_SHOW_HASH_ENABLED("HAVAL", HAVAL_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("HAVAL3", HAVAL3_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("HAVAL4", HAVAL4_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("HAVAL5", HAVAL5_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("MD2", MD2_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("MD4", MD4_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("MD5", MD5_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("Panama-LE Hash", PANAMA_LITTLE_ENDIAN_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("Panama-BE Hash", PANAMA_BIG_ENDIAN_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-128", RIPEMD128_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-160", RIPEMD160_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-256", RIPEMD256_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-320", RIPEMD320_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-1", SHA1_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-256", SHA256_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-384", SHA384_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-512", SHA512_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("Tiger", TIGER_HASH);
	CRYPTOPP_SHOW_HASH_ENABLED("Whirlpool", WHIRLPOOL_HASH);

	php_info_print_table_colspan_header(2, "Keyed-hash Message Authentication Code (HMAC) Algorithms");
	CRYPTOPP_SHOW_HASH_ENABLED("MD2", MD2_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("MD4", MD4_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("MD5", MD5_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-128", RIPEMD128_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-160", RIPEMD160_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-256", RIPEMD256_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("RIPEMD-320", RIPEMD320_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-1", SHA1_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-256", SHA256_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-384", SHA384_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("SHA-512", SHA512_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("Tiger", TIGER_HMAC);
	CRYPTOPP_SHOW_HASH_ENABLED("Whirlpool", WHIRLPOOL_HMAC);

	php_info_print_table_colspan_header(2, "Checksum Algorithms");
	CRYPTOPP_SHOW_HASH_ENABLED("Adler32", ADLER32_CHECKSUM);
	CRYPTOPP_SHOW_HASH_ENABLED("CRC32", CRC32_CHECKSUM);

	php_info_print_table_colspan_header(2, "Random Number Generators");
	#if defined(CRYPTOPP_WIN32_AVAILABLE) && defined(USE_MS_CRYPTOAPI)
	php_info_print_table_row(2, "Non-blocking (Microsoft CryptoAPI)",	"available");
	#elif defined(NONBLOCKING_RNG_AVAILABLE)
	php_info_print_table_row(2, "Non-blocking (/dev/urandom, etc.)",	"available");
	#endif

	#ifdef BLOCKING_RNG_AVAILABLE
	php_info_print_table_row(2, "Blocking (/dev/random, etc.)",			"available");
	#endif

	php_info_print_table_row(2, "System rand() function",				"available");

	php_info_print_table_row(2, "Default random source",
	#if defined(CRYPTOPP_WIN32_AVAILABLE) && defined(USE_MS_CRYPTOAPI)
		"Non-blocking (Microsoft CryptoAPI)"
	#elif defined(NONBLOCKING_RNG_AVAILABLE)
		"Non-blocking (/dev/urandom, etc.)"
	#elif defined(BLOCKING_RNG_AVAILABLE)
		"Blocking (/dev/random, etc.)"
	#else
		"System rand() function"
	#endif
	);

	php_info_print_table_end();
	DISPLAY_INI_ENTRIES();
}
/* }}} */


/* {{{ static bool cipherEnabled(Cipher TSRMLS_DC)
   Just quickly checks to see if something's been disabled via an ini setting or not even compiled
   in at all... */
static bool cipherEnabled(Cipher cipher TSRMLS_DC) {
	switch (cipher) {
		case ARC4_STREAM:					return (bool) (ENABLED_ARC4 && !CRYPTOPP_GET_INI(ini_disable_arc4));
		case PANAMA_LITTLE_ENDIAN_STREAM:	return (bool) (ENABLED_PANAMA_LITTLE_ENDIAN_CIPHER && !CRYPTOPP_GET_INI(ini_disable_panama_le_cipher));
		case PANAMA_BIG_ENDIAN_STREAM:		return (bool) (ENABLED_PANAMA_BIG_ENDIAN_CIPHER && !CRYPTOPP_GET_INI(ini_disable_panama_be_cipher));
		case MARC4_STREAM:					return (bool) (ENABLED_MARC4 && !CRYPTOPP_GET_INI(ini_disable_marc4));
		case SEAL_LITTLE_ENDIAN_STREAM:		return (bool) (ENABLED_SEAL_LITTLE_ENDIAN && !CRYPTOPP_GET_INI(ini_disable_seal_le));
		case SEAL_BIG_ENDIAN_STREAM:		return (bool) (ENABLED_SEAL_BIG_ENDIAN && !CRYPTOPP_GET_INI(ini_disable_seal_be));

		case THREEWAY_CIPHER:				return (bool) (ENABLED_3WAY && !CRYPTOPP_GET_INI(ini_disable_3way));
		case AES_CIPHER:					return (bool) (ENABLED_AES && !CRYPTOPP_GET_INI(ini_disable_aes));
		case BLOWFISH_CIPHER:				return (bool) (ENABLED_BLOWFISH && !CRYPTOPP_GET_INI(ini_disable_blowfish));
		case CAMELLIA_CIPHER:				return (bool) (ENABLED_CAMELLIA && !CRYPTOPP_GET_INI(ini_disable_camellia));
		case CAST128_CIPHER:				return (bool) (ENABLED_CAST128 && !CRYPTOPP_GET_INI(ini_disable_cast128));
		case CAST256_CIPHER:				return (bool) (ENABLED_CAST256 && !CRYPTOPP_GET_INI(ini_disable_cast256));
		case DES_CIPHER:					return (bool) (ENABLED_DES && !CRYPTOPP_GET_INI(ini_disable_des));
		case DES_EDE2_CIPHER:				return (bool) (ENABLED_DES_EDE2 && !CRYPTOPP_GET_INI(ini_disable_des_ede2));
		case DES_EDE3_CIPHER:				return (bool) (ENABLED_DES_EDE3 && !CRYPTOPP_GET_INI(ini_disable_des_ede3));
		case DES_XEX3_CIPHER:				return (bool) (ENABLED_DES_XEX3 && !CRYPTOPP_GET_INI(ini_disable_des_xex3));
		case DIAMOND2_CIPHER:				return (bool) (ENABLED_DIAMOND2 && !CRYPTOPP_GET_INI(ini_disable_diamond2));
		case DIAMOND2_LITE_CIPHER:			return (bool) (ENABLED_DIAMOND2_LITE && !CRYPTOPP_GET_INI(ini_disable_diamond2_lite));
		case GOST_CIPHER:					return (bool) (ENABLED_GOST && !CRYPTOPP_GET_INI(ini_disable_gost));
		case IDEA_CIPHER:					return (bool) (ENABLED_IDEA && !CRYPTOPP_GET_INI(ini_disable_idea));
		case MARS_CIPHER:					return (bool) (ENABLED_MARS && !CRYPTOPP_GET_INI(ini_disable_mars));
		case RC2_CIPHER:					return (bool) (ENABLED_RC2 && !CRYPTOPP_GET_INI(ini_disable_rc2));
		case RC5_CIPHER:					return (bool) (ENABLED_RC5 && !CRYPTOPP_GET_INI(ini_disable_rc5));
		case RC6_CIPHER:					return (bool) (ENABLED_RC6 && !CRYPTOPP_GET_INI(ini_disable_rc6));
		case SAFER_K_CIPHER:				return (bool) (ENABLED_SAFER_K && !CRYPTOPP_GET_INI(ini_disable_safer_k));
		case SAFER_SK_CIPHER:				return (bool) (ENABLED_SAFER_SK && !CRYPTOPP_GET_INI(ini_disable_safer_sk));
		case SHACAL2_CIPHER:				return (bool) (ENABLED_SHACAL2 && !CRYPTOPP_GET_INI(ini_disable_shacal2));
		case SHARK_CIPHER:					return (bool) (ENABLED_SHARK && !CRYPTOPP_GET_INI(ini_disable_shark));
		case SERPENT_CIPHER:				return (bool) (ENABLED_SERPENT && !CRYPTOPP_GET_INI(ini_disable_serpent));
		case SKIPJACK_CIPHER:				return (bool) (ENABLED_SKIPJACK && !CRYPTOPP_GET_INI(ini_disable_skipjack));
		case SQUARE_CIPHER:					return (bool) (ENABLED_SQUARE && !CRYPTOPP_GET_INI(ini_disable_square));
		case TEA_CIPHER:					return (bool) (ENABLED_TEA && !CRYPTOPP_GET_INI(ini_disable_tea));
		case TWOFISH_CIPHER:				return (bool) (ENABLED_TWOFISH && !CRYPTOPP_GET_INI(ini_disable_twofish));
	}
	return false;
}
/* }}} */


/* {{{ static bool hashEnabled(Hash TSRMLS_DC)
   Just quickly checks to see if somethings been disabled via an ini setting or not even compiled
   in at all... */
static bool hashEnabled(Hash hash TSRMLS_DC) {
	switch (hash) {
		case ADLER32_CHECKSUM:				return (bool) (ENABLED_ADLER32 && !CRYPTOPP_GET_INI(ini_disable_adler32));
		case CRC32_CHECKSUM:				return (bool) (ENABLED_CRC32 && !CRYPTOPP_GET_INI(ini_disable_crc32));

		case HAVAL_HASH:					return (bool) (ENABLED_HAVAL && !CRYPTOPP_GET_INI(ini_disable_haval));
		case HAVAL3_HASH:					return (bool) (ENABLED_HAVAL3 && !CRYPTOPP_GET_INI(ini_disable_haval3));
		case HAVAL4_HASH:					return (bool) (ENABLED_HAVAL4 && !CRYPTOPP_GET_INI(ini_disable_haval4));
		case HAVAL5_HASH:					return (bool) (ENABLED_HAVAL5 && !CRYPTOPP_GET_INI(ini_disable_haval5));
		case MD2_HASH:						return (bool) (ENABLED_MD2 && !CRYPTOPP_GET_INI(ini_disable_md2));
		case MD4_HASH:						return (bool) (ENABLED_MD4 && !CRYPTOPP_GET_INI(ini_disable_md4));
		case MD5_HASH:						return (bool) (ENABLED_MD5 && !CRYPTOPP_GET_INI(ini_disable_md5));
		case PANAMA_LITTLE_ENDIAN_HASH:		return (bool) (ENABLED_PANAMA_LITTLE_ENDIAN_HASH && !CRYPTOPP_GET_INI(ini_disable_panama_le_hash));
		case PANAMA_BIG_ENDIAN_HASH:		return (bool) (ENABLED_PANAMA_BIG_ENDIAN_HASH && !CRYPTOPP_GET_INI(ini_disable_panama_be_hash));
		case RIPEMD128_HASH:				return (bool) (ENABLED_RIPEMD128 && !CRYPTOPP_GET_INI(ini_disable_ripemd128));
		case RIPEMD160_HASH:				return (bool) (ENABLED_RIPEMD160 && !CRYPTOPP_GET_INI(ini_disable_ripemd160));
		case RIPEMD256_HASH:				return (bool) (ENABLED_RIPEMD256 && !CRYPTOPP_GET_INI(ini_disable_ripemd256));
		case RIPEMD320_HASH:				return (bool) (ENABLED_RIPEMD320 && !CRYPTOPP_GET_INI(ini_disable_ripemd320));
		case SHA1_HASH:						return (bool) (ENABLED_SHA1 && !CRYPTOPP_GET_INI(ini_disable_sha1));
		case SHA256_HASH:					return (bool) (ENABLED_SHA256 && !CRYPTOPP_GET_INI(ini_disable_sha256));
		case SHA384_HASH:					return (bool) (ENABLED_SHA384 && !CRYPTOPP_GET_INI(ini_disable_sha384));
		case SHA512_HASH:					return (bool) (ENABLED_SHA512 && !CRYPTOPP_GET_INI(ini_disable_sha512));
		case TIGER_HASH:					return (bool) (ENABLED_TIGER && !CRYPTOPP_GET_INI(ini_disable_tiger));
		case WHIRLPOOL_HASH:				return (bool) (ENABLED_WHIRLPOOL && !CRYPTOPP_GET_INI(ini_disable_whirlpool));

		case MD2_HMAC:						return (bool) (ENABLED_MD2_HMAC && !CRYPTOPP_GET_INI(ini_disable_md2_hmac));
		case MD4_HMAC:						return (bool) (ENABLED_MD4_HMAC && !CRYPTOPP_GET_INI(ini_disable_md4_hmac));
		case MD5_HMAC:						return (bool) (ENABLED_MD5_HMAC && !CRYPTOPP_GET_INI(ini_disable_md5_hmac));
		case RIPEMD128_HMAC:				return (bool) (ENABLED_RIPEMD128_HMAC && !CRYPTOPP_GET_INI(ini_disable_ripemd128_hmac));
		case RIPEMD160_HMAC:				return (bool) (ENABLED_RIPEMD160_HMAC && !CRYPTOPP_GET_INI(ini_disable_ripemd160_hmac));
		case RIPEMD256_HMAC:				return (bool) (ENABLED_RIPEMD256_HMAC && !CRYPTOPP_GET_INI(ini_disable_ripemd256_hmac));
		case RIPEMD320_HMAC:				return (bool) (ENABLED_RIPEMD320_HMAC && !CRYPTOPP_GET_INI(ini_disable_ripemd320_hmac));
		case SHA1_HMAC:						return (bool) (ENABLED_SHA1_HMAC && !CRYPTOPP_GET_INI(ini_disable_sha1_hmac));
		case SHA256_HMAC:					return (bool) (ENABLED_SHA256_HMAC && !CRYPTOPP_GET_INI(ini_disable_sha256_hmac));
		case SHA384_HMAC:					return (bool) (ENABLED_SHA384_HMAC && !CRYPTOPP_GET_INI(ini_disable_sha384_hmac));
		case SHA512_HMAC:					return (bool) (ENABLED_SHA512_HMAC && !CRYPTOPP_GET_INI(ini_disable_sha512_hmac));
		case TIGER_HMAC:					return (bool) (ENABLED_TIGER_HMAC && !CRYPTOPP_GET_INI(ini_disable_tiger_hmac));
		case WHIRLPOOL_HMAC:				return (bool) (ENABLED_WHIRLPOOL_HMAC && !CRYPTOPP_GET_INI(ini_disable_whirlpool_hmac));
	}
	return false;
}
/* }}} */


/* {{{ proto resource cryptopp_create_cipher(long cipher_const)
   Returns a resource to a Crypto++ cipher. cipher_const is the constant that refers
   to a specific cipher, i.e. CRYPTOPP_CIPHER_BLOWFISH, CRYPTOPP_CIPHER_3WAY, etc. */
PHP_FUNCTION(cryptopp_create_cipher)
{
	JBase* cipher = NULL;
	long algorithm = -1;
	CRYPTOPP_GET_THIS();
	CRYPTOPP_SET_ERROR_HANDLING();

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &algorithm) == FAILURE) {
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else if (!VALID_CIPHER(algorithm)) {
		CRYPTOPP_THROW_ARG(E_WARNING, "%s() received a bad cipher algorithm", get_active_function_name(TSRMLS_C));
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else if (!cipherEnabled((Cipher) algorithm TSRMLS_CC)) {
		CRYPTOPP_THROW_ARG(E_WARNING, "%s() can't create a cipher resource because the requested algorithm has been disabled", get_active_function_name(TSRMLS_C));
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else {
		switch (algorithm) {
			#if ENABLED_ARC4
			case ARC4_STREAM:
				cipher = static_cast<JARC4*>(new JARC4);
			break;
			#endif

			#if ENABLED_MARC4
			case MARC4_STREAM:
				cipher = static_cast<JMARC4*>(new JMARC4);
			break;
			#endif

			#if ENABLED_PANAMA_LITTLE_ENDIAN_CIPHER
			case PANAMA_LITTLE_ENDIAN_STREAM:
				cipher = static_cast<JPanamaCipherLE*>(new JPanamaCipherLE);
			break;
			#endif

			#if ENABLED_PANAMA_BIG_ENDIAN_CIPHER
			case PANAMA_BIG_ENDIAN_STREAM:
				cipher = static_cast<JPanamaCipherBE*>(new JPanamaCipherBE);
			break;
			#endif

			#if ENABLED_SEAL_LITTLE_ENDIAN
			case SEAL_LITTLE_ENDIAN_STREAM:
				cipher = static_cast<JSEAL_LE*>(new JSEAL_LE);
			break;
			#endif

			#if ENABLED_SEAL_BIG_ENDIAN
			case SEAL_BIG_ENDIAN_STREAM:
				cipher = static_cast<JSEAL_BE*>(new JSEAL_BE);
			break;
			#endif


			#if ENABLED_3WAY
			case THREEWAY_CIPHER:
				cipher = static_cast<J3Way*>(new J3Way);
			break;
			#endif

			#if ENABLED_AES
			case AES_CIPHER:
				cipher = static_cast<JAES*>(new JAES);
			break;
			#endif

			#if ENABLED_BLOWFISH
			case BLOWFISH_CIPHER:
				cipher = static_cast<JBlowfish*>(new JBlowfish);
			break;
			#endif

			#if ENABLED_CAMELLIA
			case CAMELLIA_CIPHER:
				cipher = static_cast<JCamellia*>(new JCamellia);
			break;
			#endif

			#if ENABLED_CAST128
			case CAST128_CIPHER:
				cipher = static_cast<JCAST128*>(new JCAST128);
			break;
			#endif

			#if ENABLED_CAST256
			case CAST256_CIPHER:
				cipher = static_cast<JCAST256*>(new JCAST256);
			break;
			#endif

			#if ENABLED_DES
			case DES_CIPHER:
				cipher = static_cast<JDES*>(new JDES);
			break;
			#endif

			#if ENABLED_DES_EDE2
			case DES_EDE2_CIPHER:
				cipher = static_cast<JDES_EDE2*>(new JDES_EDE2);
			break;
			#endif

			#if ENABLED_DES_EDE3
			case DES_EDE3_CIPHER:
				cipher = static_cast<JDES_EDE3*>(new JDES_EDE3);
			break;
			#endif

			#if ENABLED_DES_XEX3
			case DES_XEX3_CIPHER:
				cipher = static_cast<JDES_XEX3*>(new JDES_XEX3);
			break;
			#endif

			#if ENABLED_DIAMOND2
			case DIAMOND2_CIPHER:
				cipher = static_cast<JDiamond2*>(new JDiamond2);
			break;
			#endif

			#if ENABLED_DIAMOND2_LITE
			case DIAMOND2_LITE_CIPHER:
				cipher = static_cast<JDiamond2Lite*>(new JDiamond2Lite);
			break;
			#endif

			#if ENABLED_GOST
			case GOST_CIPHER:
				cipher = static_cast<JGOST*>(new JGOST);
			break;
			#endif

			#if ENABLED_IDEA
			case IDEA_CIPHER:
				cipher = static_cast<JIDEA*>(new JIDEA);
			break;
			#endif

			#if ENABLED_MARS
			case MARS_CIPHER:
				cipher = static_cast<JMARS*>(new JMARS);
			break;
			#endif

			#if ENABLED_RC2
			case RC2_CIPHER:
				cipher = static_cast<JRC2*>(new JRC2);
			break;
			#endif

			#if ENABLED_RC5
			case RC5_CIPHER:
				cipher = static_cast<JRC5*>(new JRC5);
			break;
			#endif

			#if ENABLED_RC6
			case RC6_CIPHER:
				cipher = static_cast<JRC6*>(new JRC6);
			break;
			#endif

			#if ENABLED_SAFER_K
			case SAFER_K_CIPHER:
				cipher = static_cast<JSAFER_K*>(new JSAFER_K);
			break;
			#endif

			#if ENABLED_SAFER_SK
			case SAFER_SK_CIPHER:
				cipher = static_cast<JSAFER_SK*>(new JSAFER_SK);
			break;
			#endif

			#if ENABLED_SERPENT
			case SERPENT_CIPHER:
				cipher = static_cast<JSerpent*>(new JSerpent);
			break;
			#endif

			#if ENABLED_SHACAL2
			case SHACAL2_CIPHER:
				cipher = static_cast<JSHACAL2*>(new JSHACAL2);
			break;
			#endif

			#if ENABLED_SHARK
			case SHARK_CIPHER:
				cipher = static_cast<JSHARK*>(new JSHARK);
			break;
			#endif

			#if ENABLED_SKIPJACK
			case SKIPJACK_CIPHER:
				cipher = static_cast<JSKIPJACK*>(new JSKIPJACK);
			break;
			#endif

			#if ENABLED_SQUARE
			case SQUARE_CIPHER:
				cipher = static_cast<JSquare*>(new JSquare);
			break;
			#endif

			#if ENABLED_TEA
			case TEA_CIPHER:
				cipher = static_cast<JTEA*>(new JTEA);
			break;
			#endif

			#if ENABLED_TWOFISH
			case TWOFISH_CIPHER:
				cipher = static_cast<JTwofish*>(new JTwofish);
			break;
			#endif
		}

		if (cipher == NULL) {
			CRYPTOPP_THROW_ARG(E_WARNING, "%s() couldn't create a new cipher resource", get_active_function_name(TSRMLS_C));
			CRYPTOPP_RESTORE_ERROR_HANDLING();
			RETURN_FALSE;
		}
		else {
			CRYPTOPP_REGISTER_CIPHER_RESOURCE(return_value, cipher, le_cryptopp_cipher);
		}
	}
	CRYPTOPP_RESTORE_ERROR_HANDLING();
}
/* }}} */


/* {{{ proto void cryptopp_destroy_cipher(resource cipher)
   Destroys a Crypto++ cipher object. */
PHP_FUNCTION(cryptopp_destroy_cipher)
{
	zval* resource;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &resource) == FAILURE) {
		RETURN_FALSE;
	}

	zend_list_delete(Z_RESVAL_P(resource));
}
/* }}} */


/* {{{ proto bool cryptopp_set_rand_iv(resource cipher, long sizeOfIV)
   Creates a random IV of length sizeOfIV. The random bits are culled from either /dev/urandom,
   /dev/random, the Microsoft CryptoAPI or the system's rand() function, which is seeded using
   time(NULL). By default, /dev/urandom is prefered on systems that support it, followed by
   /dev/random, then finally the system rand() function if all else fails. If the MS CryptoAPI
   is available on Windows systems, use it. */
PHP_FUNCTION(cryptopp_set_rand_iv)
{
	zval* resource;
	JBase* cipher = NULL;
	unsigned int sizeOfIV;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &sizeOfIV))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &sizeOfIV))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set IV on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set IV on stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		((JCipher*) cipher)->setRandIV(sizeOfIV);
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto bool cryptopp_set_iv(resource cipher, string iv [, bool bin])
   Set the IV of the cipher. By default, the IV is expected to be in double-digit hex;
   set bin to true to pass the IV in binary. Returns true on success, false on error. */
PHP_FUNCTION(cryptopp_set_iv)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool bin = 0;
	char* iv;
	int len = -1;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &iv, &len, &bin))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &resource, &iv, &len, &bin))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set IV on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set IV on stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		((JCipher*) cipher)->setIV(string(iv, len), !bin);
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_iv(resource cipher [, bool bin])
   Return the IV of the cipher. By default, the IV is returned in upper-case double-digit
   hex. Set bin to true to return the string in binary. */
PHP_FUNCTION(cryptopp_get_iv)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool bin = 0;
	string retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &bin))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|b", &resource, &bin))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get IV on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't get IV from stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = ((JCipher*) cipher)->getIV(!bin);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto bool cryptopp_set_mode(resource cipher, int mode)
   Set the mode of the cipher using the constants CRYPTOPP_MODE_CBC, CRYPTOPP_MODE_ECB, etc.
   Returns true on success, false on error. */
PHP_FUNCTION(cryptopp_set_mode)
{
	zval* resource;
	JBase* cipher = NULL;
	int mode = -1;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &mode))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &mode))

	if (mode < ECB_MODE || mode > OFB_MODE) {
		php_error(E_WARNING, "invalid cipher mode in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (cipher == NULL) {
		php_error(E_WARNING, "can't set mode on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set mode on stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		((JCipher*) cipher)->setMode((enum Mode) mode);
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_mode(resource cipher)
   Return the current mode of the cipher, or false on error. */
PHP_FUNCTION(cryptopp_get_mode)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get mode on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't get mode from stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(((JCipher*) cipher)->getMode());
	}
}
/* }}} */


/* {{{ proto long cryptopp_set_padding(resource cipher, int padding)
   Set the padding of the cipher using the constants CRYPTOPP_PAD_ZEROES, etc.
   Since not all modes can use all types of padding, the actual padding being
   used is returned. Returns false on error, i.e. if you try to set the padding
   on a stream cipher. */
PHP_FUNCTION(cryptopp_set_padding)
{
	zval* resource;
	JBase* cipher = NULL;
	int padding = -1;
	enum Padding retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &padding))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &padding))

	if (padding < NO_PADDING || padding > DEFAULT_PADDING) {
		php_error(E_WARNING, "invalid cipher mode in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (cipher == NULL) {
		php_error(E_WARNING, "can't set padding on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set padding on stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = ((JCipher*) cipher)->setPadding((enum Padding) padding);

		if (retval != (enum Padding) padding) {
			php_error(E_WARNING, "tried to set padding as %d, had to use %d instead in %s()", padding, retval, get_active_function_name(TSRMLS_C));
		}
		RETVAL_LONG(retval);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_padding(resource cipher)
   Return the current padding of the cipher, or false on error. */
PHP_FUNCTION(cryptopp_get_padding)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get padding on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't get padding from stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(((JCipher*) cipher)->getPadding());
	}
}
/* }}} */


/* {{{ proto long cryptopp_set_rng(resource cipher, int rng)
   Set the random number generator that the block cipher uses for generating initialization
   vectors. Returns false on error, i.e. if you try to set the RNG on a stream cipher.
   Since not all types of RNGs will be available on all systems, the actual RNG used
   will be returned on success. */
PHP_FUNCTION(cryptopp_set_rng)
{
	zval* resource;
	JBase* cipher = NULL;
	int rng = -1;
	enum RNG retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &rng))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &rng))

	if (rng < NONBLOCKING_RNG || rng > RAND_RNG) {
		php_error(E_WARNING, "invalid RNG in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (cipher == NULL) {
		php_error(E_WARNING, "can't set RNG on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set RNG on stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = ((JCipher*) cipher)->setRNG((enum RNG) rng);

		if (retval != (enum RNG) rng) {
			php_error(E_WARNING, "tried to set RNG as %d, had to use %d instead in %s()", rng, retval, get_active_function_name(TSRMLS_C));
		}
		RETVAL_LONG(retval);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_rng(resource cipher)
	Return the current random number generator of the cipher, or false on error. */
PHP_FUNCTION(cryptopp_get_rng)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get RNG on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't get RNG from stream cipher in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(((JCipher*) cipher)->getRNG());
	}
}
/* }}} */


/* {{{ proto void cryptopp_srand(long seed)
	Just an alias for PHP's own srand(). MSVC doesn't seem to like it when you
	do a PHP_FALIAS() in here... linker errors and whatnot... */
PHP_FUNCTION(cryptopp_srand)
{
	long seed;
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &seed) == FAILURE) {
		return;
	}

	if (ZEND_NUM_ARGS() == 0)
		seed = GENERATE_SEED();

	php_srand(seed TSRMLS_CC);
}
/* }}} */


/* {{{ proto bool cryptopp_set_plaintext(resource cipher, string plaintext [, bool hex])
	Set the plaintext of the cipher. Returns true on success, false on error. The default is to
	set the plaintext as binary, but you can do it as hex if you set hex to true. The length is
	of the plaintext is automatically determined. */
PHP_FUNCTION(cryptopp_set_plaintext)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool hex = 0;
	int len = -1;
	char* plaintext;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &plaintext, &len, &hex))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &resource, &plaintext, &len, &hex))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set plaintext on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		cipher->setPlaintext(string(plaintext, len), (hex ? true : false));
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_plaintext(resource cipher [, bool hex])
	Returns the plaintext of the cipher. Returns false on error. If hex it set to true,
	the string returned is in hex; otherwise, the string is in plain ol' binary. */
PHP_FUNCTION(cryptopp_get_plaintext)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool hex = 0;
	string retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &hex))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|b", &resource, &hex))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get plaintext from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = cipher->getPlaintext(hex ? true : false);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto bool cryptopp_set_ciphertext(resource cipher, string ciphertext [, bool bin])
	Set the ciphertext of the cipher. Returns true on success, false on error. The default is to
	set the ciphertext as hex, but you can do it as binary if you set bin to true. The length is
	of the ciphertext is automatically determined. */
PHP_FUNCTION(cryptopp_set_ciphertext)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool bin = 0;
	int len = -1;
	char* ciphertext;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &ciphertext, &len, &bin))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &resource, &ciphertext, &len, &bin))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set ciphertext on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		cipher->setCiphertext(string(ciphertext, len), !bin);
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_ciphertext(resource cipher [, bool bin])
	Returns the ciphertext of the cipher. Returns false on error. If bin it set to true,
	the string returned is in binary; otherwise, the string is in hex. */
PHP_FUNCTION(cryptopp_get_ciphertext)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool bin = 0;
	string retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &bin))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|b", &resource, &bin))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get ciphertext from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = cipher->getCiphertext(!bin);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_key(resource cipher, string key [, bool hex])
	Set the secret key of the cipher. The default is to set the key as binary, but you can do it
	as hex if you set hex to true. The length of the key is automatically determined. Returns the
	actual length of the key on success, false on error. The true length of the key might not be
	what you expect, as different algorithms behave differently, i.e. 3Way has a fixed keylength of
	12 bytes, while Blowfish can use keys of 1 to 72 bytes. */
PHP_FUNCTION(cryptopp_set_key)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool hex = 0;
	int len = -1;
	char* key;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &key, &len, &hex))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &resource, &key, &len, &hex))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set key on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		cipher->setKey(string(key, len), (hex ? true : false));
		RETVAL_LONG(cipher->getKeylength());
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_key(resource cipher [, bool hex])
	Returns the key of the cipher. Returns false on error. If hex it set to true,
	the string returned is in hex; otherwise, the string is in binary. */
PHP_FUNCTION(cryptopp_get_key)
{
	zval* resource;
	JBase* cipher = NULL;
	zend_bool hex = 0;
	string retval;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &hex))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|b", &resource, &hex))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get key from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = cipher->getKey(hex ? true : false);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_key_length(resource cipher, int keylength)
	Set the length of the key. Returns the true keylength on success, false on error. Depending
	on the algorithm being used, the keylength you enter may not necessarily be the keylength that is
	set -- for instance, 3Way uses a fixed-length 12 bit key, while Blowfish uses keys of 1 to 72
	bits. */
PHP_FUNCTION(cryptopp_set_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	unsigned int keylength, actual;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &keylength))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &keylength))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set keylength on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		actual = cipher->setKeylength(keylength);

		if (actual != keylength) {
			php_error(E_WARNING, "%s() set keylength to %d, but the requested length was %d", get_active_function_name(TSRMLS_C), actual, keylength);
		}
		RETVAL_LONG(actual);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_key_length(resource cipher)
	Returns the length of the secret key. The length returned is for the key in BINARY, not hex. To figure
	out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getKeylength());
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_default_key_length(resource cipher)
	Returns the default length of the secret key. The length returned is for the key in BINARY, not hex.
	To figure out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_default_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get default keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getDefaultKeylength());
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_min_key_length(resource cipher)
	Returns the minimum length of the secret key. The length returned is for the key in BINARY, not hex.
	To figure out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_min_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get minimum keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getMinKeylength());
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_max_key_length(resource cipher)
	Returns the maximum length of the secret key. The length returned is for the key in BINARY, not hex.
	To figure out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_max_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get maximum keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getMaxKeylength());
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_mult_key_length(resource cipher)
	Returns the multiplier of the secret key length. The multiplier returned is for the key in BINARY, not hex.
	To figure out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_mult_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get keylength multiplier from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getMultKeylength());
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_valid_key_length(resource cipher, int keylength)
	Returns the closest valid keylength. The length returned is for the key in BINARY, not hex.
	To figure out the hex, obviously just multiply the result by two. Returns false on error. This is
	just like cryptopp_set_key_length(), except it doesn't actually set the keylength. */
PHP_FUNCTION(cryptopp_get_valid_key_length)
{
	zval* resource;
	JBase* cipher = NULL;
	unsigned int keylength;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &keylength))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &keylength))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get valid keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getValidKeylength(keylength));
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_block_size(resource cipher)
	Returns the cipher's block size. This is obviously done in binary, not hex. */
PHP_FUNCTION(cryptopp_get_block_size)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get block size from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getBlockSize());
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_rounds(resource cipher, int rounds)
	Set the rounds used by the cipher. Returns the true rounds on success, false on error. Depending
	on the algorithm being used, the rounds you enter may not necessarily be the value that is
	actually set -- for instance, RC6 requires a minimum of 8 rounds. With some algorithms, using
	this function doesn't do anything. See documentation for details. */
PHP_FUNCTION(cryptopp_set_rounds)
{
	zval* resource;
	JBase* cipher = NULL;
	unsigned int rounds, actual;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &rounds))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &rounds))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't set rounds on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't set rounds on stream ciphers in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		((JCipher*) cipher)->setRounds(rounds);
		actual = ((JCipher*) cipher)->getRounds();

		if (actual != rounds) {
			php_error(E_WARNING, "%s() set rounds to %d, but the requested number of rounds was %d", get_active_function_name(TSRMLS_C), actual, rounds);
		}
		RETVAL_LONG(actual);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_roundsh(resource cipher)
	Returns the rounds used by the cipher. Returns false on error. With some ciphers, this is meaning less, as they
	have a fixed number of rounds.  */
PHP_FUNCTION(cryptopp_get_rounds)
{
	zval* resource;
	JBase* cipher = NULL;
	int rounds = -1;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get rounds on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_STREAM_CIPHER(cipher->getCipherType())) {
		php_error(E_WARNING, "can't get rounds from stream ciphers in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(((JCipher*) cipher)->getRounds());
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_cipher_name(mixed arg)
	Returns the name of the cipher being used. If arg is a cipher resource, it returns the
	name of the cipher being used by said resource. If the arg is one of the available
	cipher constants, it returns that name. Returns false on error. */
PHP_FUNCTION(cryptopp_get_cipher_name)
{
	zval* arg = NULL;
	JBase* cipher = NULL;
	int cipherType = -1;
	string retval;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &arg) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_PP(&arg) == IS_LONG) {
			convert_to_long_ex(&arg);
		}
		else if (Z_TYPE_PP(&arg) == IS_RESOURCE) {
			ZEND_FETCH_RESOURCE(cipher, JBase*, &arg, -1, "cryptopp cipher", le_cryptopp_cipher);
		}
	}

	if (cipher != NULL) {
		cipherType = cipher->getCipherType();
	}
	else if (arg != NULL) {
		cipherType = arg->value.lval;
	}
	else {
		php_error(E_WARNING, "can't get cipher type from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	switch (cipherType) {
		default:
			php_error(E_WARNING, "%s() could not find a valid cipher type", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		break;

		#if ENABLED_ARC4
		case ARC4_STREAM:
			retval = JARC4::getStaticCipherName();
		break;
		#endif

		#if ENABLED_MARC4
		case MARC4_STREAM:
			retval = JMARC4::getStaticCipherName();
		break;
		#endif

		#if ENABLED_PANAMA_LITTLE_ENDIAN_CIPHER
		case PANAMA_LITTLE_ENDIAN_STREAM:
			retval = JPanamaCipherLE::getStaticCipherName();
		break;
		#endif

		#if ENABLED_PANAMA_BIG_ENDIAN_CIPHER
		case PANAMA_BIG_ENDIAN_STREAM:
			retval = JPanamaCipherBE::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SEAL_LITTLE_ENDIAN
		case SEAL_LITTLE_ENDIAN_STREAM:
			retval = JSEAL_LE::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SEAL_BIG_ENDIAN
		case SEAL_BIG_ENDIAN_STREAM:
			retval = JSEAL_BE::getStaticCipherName();
		break;
		#endif

		#if ENABLED_3WAY
		case THREEWAY_CIPHER:
			retval = J3Way::getStaticCipherName();
		break;
		#endif

		#if ENABLED_AES
		case AES_CIPHER:
			retval = JAES::getStaticCipherName();
		break;
		#endif

		#if ENABLED_BLOWFISH
		case BLOWFISH_CIPHER:
			retval = JBlowfish::getStaticCipherName();
		break;
		#endif

		#if ENABLED_CAMELLIA
		case CAMELLIA_CIPHER:
			retval = JCamellia::getStaticCipherName();
		break;
		#endif

		#if ENABLED_CAST128
		case CAST128_CIPHER:
			retval = JCAST128::getStaticCipherName();
		break;
		#endif

		#if ENABLED_CAST256
		case CAST256_CIPHER:
			retval = JCAST256::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DES
		case DES_CIPHER:
			retval = JDES::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DES_EDE2
		case DES_EDE2_CIPHER:
			retval = JDES_EDE2::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DES_EDE3
		case DES_EDE3_CIPHER:
			retval = JDES_EDE3::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DES_XEX3
		case DES_XEX3_CIPHER:
			retval = JDES_XEX3::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DIAMOND2
		case DIAMOND2_CIPHER:
			retval = JDiamond2::getStaticCipherName();
		break;
		#endif

		#if ENABLED_DIAMOND2_LITE
		case DIAMOND2_LITE_CIPHER:
			retval = JDiamond2Lite::getStaticCipherName();
		break;
		#endif

		#if ENABLED_GOST
		case GOST_CIPHER:
			retval = JGOST::getStaticCipherName();
		break;
		#endif

		#if ENABLED_IDEA
		case IDEA_CIPHER:
			retval = JIDEA::getStaticCipherName();
		break;
		#endif

		#if ENABLED_MARS
		case MARS_CIPHER:
			retval = JMARS::getStaticCipherName();
		break;
		#endif

		#if ENABLED_RC2
		case RC2_CIPHER:
			retval = JRC2::getStaticCipherName();
		break;
		#endif

		#if ENABLED_RC5
		case RC5_CIPHER:
			retval = JRC5::getStaticCipherName();
		break;
		#endif

		#if ENABLED_RC6
		case RC6_CIPHER:
			retval = JRC6::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SAFER_K
		case SAFER_K_CIPHER:
			retval = JSAFER_K::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SAFER_SK
		case SAFER_SK_CIPHER:
			retval = JSAFER_SK::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SERPENT
		case SERPENT_CIPHER:
			retval = JSerpent::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SHACAL2
		case SHACAL2_CIPHER:
			retval = JSHACAL2::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SHARK
		case SHARK_CIPHER:
			retval = JSHARK::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SKIPJACK
		case SKIPJACK_CIPHER:
			retval = JSKIPJACK::getStaticCipherName();
		break;
		#endif

		#if ENABLED_SQUARE
		case SQUARE_CIPHER:
			retval = JSquare::getStaticCipherName();
		break;
		#endif

		#if ENABLED_TEA
		case TEA_CIPHER:
			retval = JTEA::getStaticCipherName();
		break;
		#endif

		#if ENABLED_TWOFISH
		case TWOFISH_CIPHER:
			retval = JTwofish::getStaticCipherName();
		break;
		#endif
	}
	RETVAL_STRINGL((char*) retval.c_str(), retval.length(), 1);
}
/* }}} */


/* {{{ proto string cryptopp_get_mode_name(mixed args)
	Returns the name of the cipher mode being used. You can use either an integer (i.e. one
	of the mode constants, like CRYPTOPP_MODE_ECB) or a cipher resource. */
PHP_FUNCTION(cryptopp_get_mode_name)
{
	zval* arg = NULL;
	JBase* cipher = NULL;
	int mode = -1;
	string retval;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &arg) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_PP(&arg) == IS_LONG) {
			convert_to_long_ex(&arg);
		}
		else if (Z_TYPE_PP(&arg) == IS_RESOURCE) {
			ZEND_FETCH_RESOURCE(cipher, JBase*, &arg, -1, "cryptopp cipher", le_cryptopp_cipher);
		}
	}

	if (cipher != NULL) {
		if (IS_STREAM_CIPHER(cipher->getCipherType())) {
			php_error(E_WARNING, "can't get mode name from stream cipher in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		mode = ((JCipher*) cipher)->getMode();
	}
	else if (arg != NULL) {
		mode = arg->value.lval;
	}
	else {
		php_error(E_WARNING, "can't get mode name from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	retval = JCipher::getModeName((enum Mode) mode);
	RETVAL_STRINGL((char*) retval.c_str(), retval.length(), 1);
}
/* }}} */


/* {{{ proto string cryptopp_get_padding_name(mixed args)
	Returns the name of the cipher padding being used. You can use either an integer (i.e. one
	of the padding constants, like CRYPTOPP_PAD_NONE) or a cipher resource. */
PHP_FUNCTION(cryptopp_get_padding_name)
{
	zval* arg = NULL;
	JBase* cipher = NULL;
	string retval;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &arg) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_PP(&arg) == IS_LONG) {
			convert_to_long_ex(&arg);
		}
		else if (Z_TYPE_PP(&arg) == IS_RESOURCE) {
			ZEND_FETCH_RESOURCE(cipher, JBase*, &arg, -1, "cryptopp cipher", le_cryptopp_cipher);
		}
	}

	if (cipher != NULL) {
		if (IS_STREAM_CIPHER(cipher->getCipherType())) {
			php_error(E_WARNING, "can't get padding name from stream cipher in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		retval = ((JCipher*) cipher)->getPaddingName();
	}
	else if (arg != NULL) {
		retval = JCipher::getPaddingName((enum Padding) arg->value.lval);
	}
	else {
		php_error(E_WARNING, "can't get padding name from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	RETVAL_STRINGL((char*) retval.c_str(), retval.length(), 1);
}
/* }}} */


/* {{{ proto string cryptopp_get_rng_name(mixed args)
	Returns the name of the random number generator being used. You can use either an
	integer (i.e. one of the RNG constants, like CRYPTOPP_RNG_BLOCKING) or a cipher resource. */
PHP_FUNCTION(cryptopp_get_rng_name)
{
	zval* arg = NULL;
	JBase* cipher = NULL;
	string retval;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &arg) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_PP(&arg) == IS_LONG) {
			convert_to_long_ex(&arg);
		}
		else if (Z_TYPE_PP(&arg) == IS_RESOURCE) {
			ZEND_FETCH_RESOURCE(cipher, JBase*, &arg, -1, "cryptopp cipher", le_cryptopp_cipher);
		}
	}

	if (cipher != NULL) {
		if (IS_STREAM_CIPHER(cipher->getCipherType())) {
			php_error(E_WARNING, "can't get RNG name from stream cipher in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		retval = ((JCipher*) cipher)->getRNGName();
	}
	else if (arg != NULL) {
		retval = JCipher::getRNGName((enum RNG) arg->value.lval);
	}
	else {
		php_error(E_WARNING, "can't get RNG name from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	RETVAL_STRINGL((char*) retval.c_str(), retval.length(), 1);
}
/* }}} */


/* {{{ proto int cryptopp_get_cipher_type(resource cipher)
	Returns the type of the cipher being used. (One of the constants above.) Returns false on error. */
PHP_FUNCTION(cryptopp_get_cipher_type)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	if (cipher == NULL) {
		php_error(E_WARNING, "can't get cipher type from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(cipher->getCipherType());
	}
}
/* }}} */


/* {{{ proto bool cryptopp_cipher_enabled(int cipher_const)
	Checks to see if a cipher algorithm is available for use, i.e. it's been
	enabled in jconfig.h. Returns true if the cipher is available, false if it isn't. */
PHP_FUNCTION(cryptopp_cipher_enabled)
{
	long algorithm = -1;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &algorithm) == FAILURE) {
		RETURN_FALSE;
	}
	else if (!VALID_CIPHER(algorithm)) {
		php_error(E_WARNING, "%s() received a bad cipher algorithm", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (cipherEnabled((Cipher) algorithm TSRMLS_CC)) {
		RETURN_TRUE;
	}
	else {
		RETURN_FALSE;
	}
}
/* }}} */


/* {{{ proto bool cryptopp_rng_available(int rng)
	Checks to see if a random number generator is available for use. Returns true if
	the RNG is available, false if it isn't. Available RNGs are determined at build-
	time. On MS Windows systems, a non-blocking RNG is made available via the
	MS CryptoAPI. On most UNIX systems, both blocking and non-blocking RNGs are
	available. The system rand() function is always available. */
PHP_FUNCTION(cryptopp_rng_available)
{
	int rng = -1;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &rng) == FAILURE) {
		RETURN_FALSE;
	}

	switch (rng) {
		#ifdef NONBLOCKING_RNG_AVAILABLE
		case NONBLOCKING_RNG:
		#endif

		#ifdef BLOCKING_RNG_AVAILABLE
		case BLOCKING_RNG:
		#endif

		case RAND_RNG:
			RETURN_TRUE;
	}

	RETURN_FALSE;
}
/* }}} */


/* {{{ proto bool cryptopp_encrypt(resource cipher)
	Encrypts the cipher's plaintext and sticks it into ciphertext, which can be retrieved
	via cryptopp_get_ciphertext(). Returns true on success, false on error. */
PHP_FUNCTION(cryptopp_encrypt)
{
	zval* resource;
	JBase* cipher = NULL;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	try {
		if (cipher == NULL) {
			php_error(E_WARNING, "can't encrypt null resource in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else if (cipher->encrypt()) {
			RETURN_TRUE;
		}
		else {
			RETURN_FALSE;
		}
	}
	catch (Exception e) {
		php_error(E_WARNING, "Crypto++ exception: %s in %s", e.GetWhat().c_str(), get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
}
/* }}} */


/* {{{ proto bool cryptopp_decrypt(resource cipher)
	Decrypts the cipher's ciphertext and sticks it into plaintext, which can be retrieved
	via cryptopp_get_plaintext(). Returns true on success, false on error. */
PHP_FUNCTION(cryptopp_decrypt)
{
	zval* resource;
	JBase* cipher = NULL;;
	CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

	CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

	try {
		if (cipher == NULL) {
			php_error(E_WARNING, "can't decrypt null resource in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else if (cipher->decrypt()) {
			RETURN_TRUE;
		}
		else {
			RETURN_FALSE;
		}
	}
	catch (Exception e) {
		php_error(E_WARNING, "Crypto++ exception: %s in %s", e.GetWhat().c_str(), get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
}
/* }}} */


/* {{{ proto bool cryptopp_encrypt_file(resource cipher, mixed inFile, mixed outFile)
	Encrypts a file (inFile) and saves it (outFile). Returns true on success, false on
	error. Both inFile and outFile can be either a filename or a previously opened
	PHP stream resource. */
PHP_FUNCTION(cryptopp_encrypt_file)
{
	zval* resource;
	JBase* cipher = NULL;
	zval *zvalInStream = NULL, *zvalOutStream = NULL;
	php_stream *inStream = NULL, *outStream = NULL;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zz", &zvalInStream, &zvalOutStream))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rzz", &resource, &zvalInStream, &zvalOutStream))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't encrypt file with null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_P(zvalInStream) == IS_STRING) {
			if (Z_STRLEN_P(zvalInStream) <= 0) {
				php_error(E_WARNING, "inFile is invalid in %s()", get_active_function_name(TSRMLS_C));
				RETURN_FALSE;
			}
			else {
				inStream = php_stream_fopen(Z_STRVAL_P(zvalInStream), "rb", NULL);
			}
		}
		else {
			php_stream_from_zval(inStream, &zvalInStream);
		}

		if (Z_TYPE_P(zvalOutStream) == IS_STRING) {
			outStream = php_stream_fopen(Z_STRVAL_P(zvalOutStream), "wb", NULL);
		}
		else {
			php_stream_from_zval(outStream, &zvalOutStream);
		}

		if (inStream == NULL) {
			php_error(E_WARNING, "can't open inFile in %s()", get_active_function_name(TSRMLS_C));
			RETVAL_FALSE;
		}
		else if (outStream == NULL) {
			php_error(E_WARNING, "can't open outFile in %s()", get_active_function_name(TSRMLS_C));
			RETVAL_FALSE;
		}
		else {
			try {
				cipher->encryptPHPStream(inStream, outStream);
				RETVAL_TRUE;
			}
			catch (Exception e) {
				php_error(E_WARNING, "Crypto++ exception:  %s in %s()", e.GetWhat().c_str(), get_active_function_name(TSRMLS_C));
				RETVAL_FALSE;
			}
		}

		if (Z_TYPE_P(zvalInStream) == IS_STRING && inStream != NULL) {
			php_stream_close(inStream);
		}
		if (Z_TYPE_P(zvalOutStream) == IS_STRING && outStream != NULL) {
			php_stream_close(outStream);
		}
	}
}
/* }}} */


/* {{{ proto bool cryptopp_decrypt_file(resource cipher, string inFile, string outFile)
	Decrypts a file (inFile) and saves it (outFile). Returns true on success, false on
	error. Both inFile and outFile can be either a filename or a previously opened
	PHP stream resource.*/
PHP_FUNCTION(cryptopp_decrypt_file)
{
	zval* resource;
	JBase* cipher = NULL;
	zval *zvalInStream = NULL, *zvalOutStream = NULL;
	php_stream *inStream = NULL, *outStream = NULL;
	CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "zz", &zvalInStream, &zvalOutStream))

	CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rzz", &resource, &zvalInStream, &zvalOutStream))

	if (cipher == NULL) {
		php_error(E_WARNING, "can't decrypt file with null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_P(zvalInStream) == IS_STRING) {
			if (Z_STRLEN_P(zvalInStream) <= 0) {
				php_error(E_WARNING, "inFile is invalid in %s()", get_active_function_name(TSRMLS_C));
				RETURN_FALSE;
			}
			else {
				inStream = php_stream_fopen(Z_STRVAL_P(zvalInStream), "rb", NULL);
			}
		}
		else {
			php_stream_from_zval(inStream, &zvalInStream);
		}

		if (Z_TYPE_P(zvalOutStream) == IS_STRING) {
			outStream = php_stream_fopen(Z_STRVAL_P(zvalOutStream), "wb", NULL);
		}
		else {
			php_stream_from_zval(outStream, &zvalOutStream);
		}

		if (inStream == NULL) {
			php_error(E_WARNING, "can't open inFile in %s()", get_active_function_name(TSRMLS_C));
			RETVAL_FALSE;
		}
		else if (outStream == NULL) {
			php_error(E_WARNING, "can't open outFile in %s()", get_active_function_name(TSRMLS_C));
			RETVAL_FALSE;
		}
		else {
			try {
				cipher->decryptPHPStream(inStream, outStream);
				RETVAL_TRUE;
			}
			catch (Exception e) {
				php_error(E_WARNING, "Crypto++ exception:  %s in %s()", e.GetWhat().c_str(), get_active_function_name(TSRMLS_C));
				RETVAL_FALSE;
			}
		}

		if (Z_TYPE_P(zvalInStream) == IS_STRING && inStream != NULL) {
			php_stream_close(inStream);
		}
		if (Z_TYPE_P(zvalOutStream) == IS_STRING && outStream != NULL) {
			php_stream_close(outStream);
		}
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_effective_key_length(resource cipher, int keylength)
   Set the effective keylength on the RC2 algorithm. This function can only be used
   with RC2. Returns the actual effective keylength set. The default is 1024, which is
   also the maximum. Returns false on error. */
PHP_FUNCTION(cryptopp_set_effective_key_length)
{
	#if ENABLED_RC2
	zval* resource;
	JBase* cipher = NULL;
	unsigned int keylength, actual;

	if (!cipherEnabled(RC2_CIPHER TSRMLS_CC)) {
		php_error(E_WARNING, "%s(): the RC2 block cipher algorithm has been disabled", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		CRYPTOPP_CIPHER_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &keylength))

		CRYPTOPP_CIPHER_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &keylength))

		if (cipher == NULL) {
			php_error(E_WARNING, "can't set keylength on null resource in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else if (cipher->getCipherType() != RC2_CIPHER) {
			php_error(E_WARNING, "%s() can only set the effective keylength on RC2 ciphers", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else {
			actual = ((JRC2*) cipher)->setEffectiveKeylength(keylength);

			if (actual != keylength) {
				php_error(E_WARNING, "%s() set effective keylength to %d, but the requested length was %d", get_active_function_name(TSRMLS_C), actual, keylength);
			}
			RETVAL_LONG(actual);
		}
	}
	#else
	php_error(E_WARNING, "%s(): the RC2 block cipher algorithm has been disabled in this cryptopp build", get_active_function_name(TSRMLS_C));
	RETURN_FALSE;
	#endif
}
/* }}} */


/* {{{ proto int cryptopp_get_effective_key_length(resource cipher)
	Returns the effective keylength of the cipher. The cipher must use the RC2 algorihtm.
	Returns false on error. */
PHP_FUNCTION(cryptopp_get_effective_key_length)
{
	#if ENABLED_RC2
	zval* resource;
	JBase* cipher = NULL;

	if (!cipherEnabled(RC2_CIPHER TSRMLS_CC)) {
		php_error(E_WARNING, "%s(): the RC2 block cipher algorithm has been disabled", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		CRYPTOPP_CIPHER_OBJ_NO_PARAMS()

		CRYPTOPP_CIPHER_RSRC_NO_PARAMS()

		if (cipher == NULL) {
			php_error(E_WARNING, "can't get keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else if (cipher->getCipherType() != RC2_CIPHER) {
			php_error(E_WARNING, "%s() can only get the effective keylength from RC2 ciphers", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else {
			RETVAL_LONG(((JRC2*) cipher)->getEffectiveKeylength());
		}
	}
	#else
	php_error(E_WARNING, "%s(): the RC2 block cipher algorithm has been disabled in this cryptopp build", get_active_function_name(TSRMLS_C));
	RETURN_FALSE;
	#endif
}
/* }}} */


// hash functions:

/* {{{ proto resource cryptopp_create_hash(int hash_const)
   Returns a resource to a Crypto++ hash. hash_const is the constant that refers
   to a specific hash algorithm, i.e. CRYPTOPP_HASH_MD2, CRYPTOPP_HASH_MD5, etc. */
PHP_FUNCTION(cryptopp_create_hash)
{
	JHash* hash = NULL;
	long algorithm = -1;
	CRYPTOPP_GET_THIS();
	CRYPTOPP_SET_ERROR_HANDLING();

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &algorithm) == FAILURE) {
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else if (algorithm < ADLER32_CHECKSUM || algorithm > WHIRLPOOL_HMAC) {
		CRYPTOPP_THROW_ARG(E_WARNING, "%s() received a bad hash algorithm", get_active_function_name(TSRMLS_C));
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else if (!hashEnabled((Hash) algorithm TSRMLS_CC)) {
		CRYPTOPP_THROW_ARG(E_WARNING, "%s() can't create a hash resource because the requested algorithm has been disabled", get_active_function_name(TSRMLS_C));
		CRYPTOPP_RESTORE_ERROR_HANDLING();
		RETURN_FALSE;
	}
	else {
		switch (algorithm) {
			#if ENABLED_ADLER32
			case ADLER32_CHECKSUM:
				hash = static_cast<JAdler32*>(new JAdler32);
			break;
			#endif

			#if ENABLED_CRC32
			case CRC32_CHECKSUM:
				hash = static_cast<JCRC32*>(new JCRC32);
			break;
			#endif

			#if ENABLED_HAVAL
			case HAVAL_HASH:
				hash = static_cast<JHAVAL*>(new JHAVAL);
			break;
			#endif

			#if ENABLED_HAVAL3
			case HAVAL3_HASH:
				hash = static_cast<JHAVAL3*>(new JHAVAL3);
			break;
			#endif

			#if ENABLED_HAVAL4
			case HAVAL4_HASH:
				hash = static_cast<JHAVAL4*>(new JHAVAL4);
			break;
			#endif

			#if ENABLED_HAVAL5
			case HAVAL5_HASH:
				hash = static_cast<JHAVAL5*>(new JHAVAL5);
			break;
			#endif

			#if ENABLED_MD2
			case MD2_HASH:
				hash = static_cast<JMD2*>(new JMD2);
			break;
			#endif

			#if ENABLED_MD4
			case MD4_HASH:
				hash = static_cast<JMD4*>(new JMD4);
			break;
			#endif

			#if ENABLED_MD5
			case MD5_HASH:
				hash = static_cast<JMD5*>(new JMD5);
			break;
			#endif

			#if ENABLED_PANAMA_LITTLE_ENDIAN_HASH
			case PANAMA_LITTLE_ENDIAN_HASH:
				hash = static_cast<JPanamaHashLE*>(new JPanamaHashLE);
			break;
			#endif

			#if ENABLED_PANAMA_BIG_ENDIAN_HASH
			case PANAMA_BIG_ENDIAN_HASH:
				hash = static_cast<JPanamaHashBE*>(new JPanamaHashBE);
			break;
			#endif

			#if ENABLED_RIPEMD128
			case RIPEMD128_HASH:
				hash = static_cast<JRIPEMD128*>(new JRIPEMD128);
			break;
			#endif

			#if ENABLED_RIPEMD160
			case RIPEMD160_HASH:
				hash = static_cast<JRIPEMD160*>(new JRIPEMD160);
			break;
			#endif

			#if ENABLED_RIPEMD256
			case RIPEMD256_HASH:
				hash = static_cast<JRIPEMD256*>(new JRIPEMD256);
			break;
			#endif

			#if ENABLED_RIPEMD320
			case RIPEMD320_HASH:
				hash = static_cast<JRIPEMD320*>(new JRIPEMD320);
			break;
			#endif

			#if ENABLED_SHA1
			case SHA1_HASH:
				hash = static_cast<JSHA1*>(new JSHA1);
			break;
			#endif

			#if ENABLED_SHA256
			case SHA256_HASH:
				hash = static_cast<JSHA256*>(new JSHA256);
			break;
			#endif

			#if ENABLED_SHA384
			case SHA384_HASH:
				hash = static_cast<JSHA384*>(new JSHA384);
			break;
			#endif

			#if ENABLED_SHA512
			case SHA512_HASH:
				hash = static_cast<JSHA512*>(new JSHA512);
			break;
			#endif

			#if ENABLED_TIGER
			case TIGER_HASH:
				hash = static_cast<JTiger*>(new JTiger);
			break;
			#endif

			#if ENABLED_WHIRLPOOL
			case WHIRLPOOL_HASH:
				hash = static_cast<JWhirlpool*>(new JWhirlpool);
			break;
			#endif

			// HMACs...

			#if ENABLED_MD2_HMAC
			case MD2_HMAC:
				hash = static_cast<JMD2_HMAC*>(new JMD2_HMAC);
			break;
			#endif

			#if ENABLED_MD4_HMAC
			case MD4_HMAC:
				hash = static_cast<JMD4_HMAC*>(new JMD4_HMAC);
			break;
			#endif

			#if ENABLED_MD5_HMAC
			case MD5_HMAC:
				hash = static_cast<JMD5_HMAC*>(new JMD5_HMAC);
			break;
			#endif

			#if ENABLED_RIPEMD128
			case RIPEMD128_HMAC:
				hash = static_cast<JRIPEMD128_HMAC*>(new JRIPEMD128_HMAC);
			break;
			#endif

			#if ENABLED_RIPEMD160
			case RIPEMD160_HMAC:
				hash = static_cast<JRIPEMD160_HMAC*>(new JRIPEMD160_HMAC);
			break;
			#endif

			#if ENABLED_RIPEMD256
			case RIPEMD256_HMAC:
				hash = static_cast<JRIPEMD256_HMAC*>(new JRIPEMD256_HMAC);
			break;
			#endif

			#if ENABLED_RIPEMD320
			case RIPEMD320_HMAC:
				hash = static_cast<JRIPEMD320_HMAC*>(new JRIPEMD320_HMAC);
			break;
			#endif

			#if ENABLED_SHA1_HMAC
			case SHA1_HMAC:
				hash = static_cast<JSHA1_HMAC*>(new JSHA1_HMAC);
			break;
			#endif

			#if ENABLED_SHA256_HMAC
			case SHA256_HMAC:
				hash = static_cast<JSHA256_HMAC*>(new JSHA256_HMAC);
			break;
			#endif

			#if ENABLED_SHA384_HMAC
			case SHA384_HMAC:
				hash = static_cast<JSHA384_HMAC*>(new JSHA384_HMAC);
			break;
			#endif

			#if ENABLED_SHA512_HMAC
			case SHA512_HMAC:
				hash = static_cast<JSHA512_HMAC*>(new JSHA512_HMAC);
			break;
			#endif

			#if ENABLED_TIGER_HMAC
			case TIGER_HMAC:
				hash = static_cast<JTiger_HMAC*>(new JTiger_HMAC);
			break;
			#endif

			#if ENABLED_WHIRLPOOL_HMAC
			case WHIRLPOOL_HMAC:
				hash = static_cast<JWhirlpool_HMAC*>(new JWhirlpool_HMAC);
			break;
			#endif
		}

		if (hash == NULL) {
			CRYPTOPP_THROW_ARG(E_WARNING, "%s() couldn't create a new hash resource", get_active_function_name(TSRMLS_C));
			CRYPTOPP_RESTORE_ERROR_HANDLING();
			RETURN_FALSE;
		}
		else {
			CRYPTOPP_REGISTER_HASH_RESOURCE(return_value, hash, le_cryptopp_hash);
		}
	}
	CRYPTOPP_RESTORE_ERROR_HANDLING();
}
/* }}} */


/* {{{ proto void cryptopp_destroy_hash(resource hash)
   Destroys a Crypto++ hash object. */
PHP_FUNCTION(cryptopp_destroy_hash)
{
	zval* resource;
	JHash* hash = NULL;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &resource) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		zend_list_delete(Z_RESVAL_P(resource));
	}
}
/* }}} */


/* {{{ proto bool cryptopp_set_hash_plaintext(resource hash, string plaintext)
	Set the plaintext of the hash. Returns true on success, false on error. Always set as
	binary. */
PHP_FUNCTION(cryptopp_set_hash_plaintext)
{
	zval* resource;
	JHash* hash = NULL;
	int len = -1;
	char* plaintext;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &plaintext, &len))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &resource, &plaintext, &len))

	if (hash == NULL) {
		php_error(E_WARNING, "can't set plaintext on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		hash->setPlaintext(string(plaintext, len));
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_hash_plaintext(resource hash)
	Returns the plaintext of the hash, or false on error. Always returned as binary. */
PHP_FUNCTION(cryptopp_get_hash_plaintext)
{
	zval* resource;
	JHash* hash = NULL;
	string retval;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	CRYPTOPP_HASH_RSRC_NO_PARAMS()

	if (hash == NULL) {
		php_error(E_WARNING, "can't get plaintext from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = hash->getPlaintext();
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto bool cryptopp_set_hashtext(resource hash, string hashtext)
	Set the hashtext of the hash. Returns true on success, false on error. Always expects the hashtext in
	hex. */
PHP_FUNCTION(cryptopp_set_hashtext)
{
	zval* resource;
	JHash* hash = NULL;
	int len = -1;
	char* hashtext;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hashtext, &len))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", &resource, &hashtext, &len))
		RETURN_FALSE;

	if (hash == NULL) {
		php_error(E_WARNING, "can't set hashtext on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		hash->setHashtext(string(hashtext, len));
		RETURN_TRUE;
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_hashtext(resource hash)
	Returns the hashtext of the hash. Returns false on error. Always returns the hashtext
	as hex. */
PHP_FUNCTION(cryptopp_get_hashtext)
{
	zval* resource;
	JHash* hash = NULL;
	string retval;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	CRYPTOPP_HASH_RSRC_NO_PARAMS()

	if (hash == NULL) {
		php_error(E_WARNING, "can't get hashtext from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = hash->getHashtext(true);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_digest_size(resource hash)
	Returns the length of the digest size of the hash. The length returned is for the digest in HEX, not binary.
	To figure out the binary, obviously divide the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_digest_size)
{
	zval* resource;
	JHash* hash = NULL;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	CRYPTOPP_HASH_RSRC_NO_PARAMS()

	if (hash == NULL) {
		php_error(E_WARNING, "can't get digest size from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(hash->getDigestSize());
	}
}
/* }}} */


/* {{{ proto bool cryptopp_hash(resource hash [, string plaintext])
	Hashes the resource's plaintext and sticks it into hashtext, which can be retrieved
	via cryptopp_get_hashtext(). Returns true on success, false on error. The plaintext
	here is optional -- if you do use a plaintext, cryptopp_hash() stores it in it's
	plaintext buffer before hashing. This basically just saves you the step of calling
	cryptopp_set_hash_plaintext() separately. */
PHP_FUNCTION(cryptopp_hash)
{
	zval* resource;
	JHash* hash = NULL;
	bool retval;
	int len;
	char* plaintext = NULL;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &plaintext, &len))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|s", &resource, &plaintext, &len))

	if (hash == NULL) {
		php_error(E_WARNING, "can't get hash null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		if (plaintext != NULL) {
			hash->setPlaintext(string(plaintext, len));
		}

		retval = hash->hash();

		if (retval) {
			RETURN_TRUE;
		}
		else {
			RETURN_FALSE;
		}
	}
}
/* }}} */


/* {{{ proto string cryptopp_hash_file(resource hash, string filename)
	Hashes the file and returns the hexadecimal message digest. Returns false
	on error. */
PHP_FUNCTION(cryptopp_hash_file)
{
	zval* resource;
	JHash* hash = NULL;
	string retval;
	zval *zvalInStream = NULL;
	php_stream *inStream = NULL;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &zvalInStream))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rz", &resource, &zvalInStream))

	if (hash == NULL) {
		php_error(E_WARNING, "can't hash file with null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_P(zvalInStream) == IS_STRING) {
			if (Z_STRLEN_P(zvalInStream) <= 0) {
				php_error(E_WARNING, "inFile is invalid in %s()", get_active_function_name(TSRMLS_C));
				RETURN_FALSE;
			}
			else {
				inStream = php_stream_fopen(Z_STRVAL_P(zvalInStream), "rb", NULL);
			}
		}
		else {
			php_stream_from_zval(inStream, &zvalInStream);
		}

		if (inStream == NULL) {
			php_error(E_WARNING, "can't open inFile in %s()", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		}
		else {
			try {
				retval = hash->hashPHPStream(inStream);
				RETVAL_STRINGL((char*) retval.c_str(), retval.length(), 1);
			}
			catch (Exception e) {
				php_error(E_WARNING, "Crypto++ exception: %s in %s()", e.GetWhat().c_str(), get_active_function_name(TSRMLS_C));
				RETVAL_FALSE;
			}
		}

		if (Z_TYPE_P(zvalInStream) == IS_STRING && inStream != NULL) {
			php_stream_close(inStream);
		}
	}
}
/* }}} */


/* {{{ proto int cryptopp_validate_hash(resource hash [, string plaintext, string hashtext])
	Checks to see if the hashtext is a valid hash of plaintext. Returns 1 on a valid hash, and 0 on a bad hash.
	May return false if the parameters given are wrong, like most PHP functions. If you specify
	a plaintext and hashtext, those are used in lieu of the hash resource's internal plaintext
	and hashtext, which will not be touched. */
PHP_FUNCTION(cryptopp_validate_hash)
{
	zval* resource;
	JHash* hash = NULL;
	char* plaintext = NULL;
	char* hashtext = NULL;
	int plainLen;
	int hashLen;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|ss", &plaintext, &plainLen, &hashtext, &hashLen))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|ss", &resource, &plaintext, &plainLen, &hashtext, &hashLen))

	#ifdef USE_PHP5_OBJECTS
	if (object && ZEND_NUM_ARGS() == 1) {
		php_error(E_WARNING, "wrong number of args, expected 0 or 2, not 1 in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else
	#endif
	if (ZEND_NUM_ARGS() == 2) {
		php_error(E_WARNING, "wrong number of args, expected 1 or 3, not 2 in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	if (hash == NULL) {
		php_error(E_WARNING, "can't get validate null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		if (plaintext != NULL && hashtext != NULL) {
			if (hash->validate(plaintext, hashtext)) {
				RETVAL_LONG(1);
			}
			else {
				RETVAL_LONG(0);
			}
		}
		else {
			if (hash->validate()) {
				RETVAL_LONG(1);
			}
			else {
				RETVAL_LONG(0);
			}
		}
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_hash_name(mixed arg)
	Returns the name of the hash being used. If arg is a hash resource, it returns the
	name of the hash being used by said resource. If the arg is one of the available
	hash constants, it returns that name. Returns false on error. */
PHP_FUNCTION(cryptopp_get_hash_name)
{
	zval* arg = NULL;
	JHash* hash = NULL;
	long hashType = -1;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &arg) == FAILURE) {
		RETURN_FALSE;
	}
	else {
		if (Z_TYPE_PP(&arg) == IS_LONG) {
			convert_to_long_ex(&arg);
		}
		else if (Z_TYPE_PP(&arg) == IS_RESOURCE) {
			ZEND_FETCH_RESOURCE(hash, JHash*, &arg, -1, "cryptopp hash", le_cryptopp_hash);
		}
	}

	if (hash != NULL) {
		hashType = hash->getHashType();
	}
	else if (arg != NULL) {
		hashType = arg->value.lval;
	}
	else {
		php_error(E_WARNING, "can't get hash name from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}

	switch (hashType) {
		default:
			php_error(E_WARNING, "%s() could not find a valid hash type", get_active_function_name(TSRMLS_C));
			RETURN_FALSE;
		break;

		#if ENABLED_ADLER32
		case ADLER32_CHECKSUM:
			RETVAL_STRING((char*) JAdler32::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_CRC32
		case CRC32_CHECKSUM:
			RETVAL_STRING((char*) JCRC32::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_HAVAL
		case HAVAL_HASH:
			RETVAL_STRING((char*) JHAVAL::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_HAVAL3
		case HAVAL3_HASH:
			RETVAL_STRING((char*) JHAVAL3::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_HAVAL4
		case HAVAL4_HASH:
			RETVAL_STRING((char*) JHAVAL4::getHashName().c_str(),1);
		break;
		#endif

		#if ENABLED_HAVAL5
		case HAVAL5_HASH:
			RETVAL_STRING((char*) JHAVAL5::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_MD2
		case MD2_HASH:
			RETVAL_STRING((char*) JMD2::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_MD4
		case MD4_HASH:
			RETVAL_STRING((char*) JMD4::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_MD5
		case MD5_HASH:
			RETVAL_STRING((char*) JMD5::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_PANAMA_LITTLE_ENDIAN_HASH
		case PANAMA_LITTLE_ENDIAN_HASH:
			RETVAL_STRING((char*) JPanamaHashLE::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_PANAMA_BIG_ENDIAN_HASH
		case PANAMA_BIG_ENDIAN_HASH:
			RETVAL_STRING((char*) JPanamaHashBE::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD128
		case RIPEMD128_HASH:
			RETVAL_STRING((char*) JRIPEMD128::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD160
		case RIPEMD160_HASH:
			RETVAL_STRING((char*) JRIPEMD160::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD256
		case RIPEMD256_HASH:
			RETVAL_STRING((char*) JRIPEMD256::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD320
		case RIPEMD320_HASH:
			RETVAL_STRING((char*) JRIPEMD320::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA1
		case SHA1_HASH:
			RETVAL_STRING((char*) JSHA1::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA256
		case SHA256_HASH:
			RETVAL_STRING((char*) JSHA256::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA384
		case SHA384_HASH:
			RETVAL_STRING((char*) JSHA384::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA512
		case SHA512_HASH:
			RETVAL_STRING((char*) JSHA512::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_TIGER
		case TIGER_HASH:
			RETVAL_STRING((char*) JTiger::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_WHIRLPOOL
		case WHIRLPOOL_HASH:
			RETVAL_STRING((char*) JWhirlpool::getHashName().c_str(), 1);
		break;
		#endif


		// HMACs...

		#if ENABLED_MD2_HMAC
		case MD2_HMAC:
			RETVAL_STRING((char*) JMD2_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_MD4_HMAC
		case MD4_HMAC:
			RETVAL_STRING((char*) JMD4_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_MD5_HMAC
		case MD5_HMAC:
			RETVAL_STRING((char*) JMD5_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD128_HMAC
		case RIPEMD128_HMAC:
			RETVAL_STRING((char*) JRIPEMD128_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD160_HMAC
		case RIPEMD160_HMAC:
			RETVAL_STRING((char*) JRIPEMD160_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD256_HMAC
		case RIPEMD256_HMAC:
			RETVAL_STRING((char*) JRIPEMD256_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_RIPEMD320_HMAC
		case RIPEMD320_HMAC:
			RETVAL_STRING((char*) JRIPEMD320_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA1_HMAC
		case SHA1_HMAC:
			RETVAL_STRING((char*) JSHA1_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA256_HMAC
		case SHA256_HMAC:
			RETVAL_STRING((char*) JSHA256_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA384_HMAC
		case SHA384_HMAC:
			RETVAL_STRING((char*) JSHA384_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_SHA512_HMAC
		case SHA512_HMAC:
			RETVAL_STRING((char*) JSHA512_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_TIGER_HMAC
		case TIGER_HMAC:
			RETVAL_STRING((char*) JTiger_HMAC::getHashName().c_str(), 1);
		break;
		#endif

		#if ENABLED_WHIRLPOOL_HMAC
		case WHIRLPOOL_HMAC:
			RETVAL_STRING((char*) JWhirlpool_HMAC::getHashName().c_str(), 1);
		break;
		#endif
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_hash_type(resource cipher)
	Returns the type of the hash being used. (One of the constants above.) Returns false on error. */
PHP_FUNCTION(cryptopp_get_hash_type)
{
	zval* resource;
	JHash* hash = NULL;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	CRYPTOPP_HASH_RSRC_NO_PARAMS()

	if (hash == NULL) {
		php_error(E_WARNING, "can't get hash type from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(hash->getHashType());
	}
}
/* }}} */


/* {{{ proto bool cryptopp_hash_enabled(int hash_const)
	Checks to see if a hash algorithm is available for use, i.e. it's been
	enabled in jconfig.h. Returns true if the hash is available, false if it isn't. */
PHP_FUNCTION(cryptopp_hash_enabled)
{
	long algorithm = -1;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &algorithm) == FAILURE) {
		RETURN_FALSE;
	}
	else if (algorithm < ADLER32_CHECKSUM || algorithm > RIPEMD320_HMAC) {
		php_error(E_WARNING, "%s() received a bad hash algorithm", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (hashEnabled((Hash) algorithm TSRMLS_CC)) {
		RETURN_TRUE;
	}
	else {
		RETURN_FALSE;
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_hmac_key(resource hmac, string key [, bool hex])
	Set the secret key of the HMAC. The default is to set the key as binary, but you can do it
	as hex if you set hex to true. The length of the key is automatically determined. Returns the
	actual length of the key on success, false on error. The true length of the key might not be
	what you expect, as different algorithms behave differently, as with the cipher function
	cryptopp_set_key(). */
PHP_FUNCTION(cryptopp_set_hmac_key)
{
	zval* resource;
	JHash* hash = NULL;
	zend_bool hex = 0;
	int len = -1;
	char* key;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &key, &len, &hex))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|b", &resource, &key, &len, &hex))

	if (hash == NULL) {
		php_error(E_WARNING, "can't set key on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_NON_HMAC(hash->getHashType())) {
		php_error(E_WARNING, "can't set a key on a regular hash, must be a HMAC hash in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		((JHMAC*) hash)->setKey(string(key, len), (hex ? true : false));
		RETVAL_LONG(((JHMAC*) hash)->getKeylength());
	}
}
/* }}} */


/* {{{ proto string cryptopp_get_hmac_key(resource hmac [, bool hex])
	Returns the key of the HMAC. Returns false on error. If hex it set to true,
	the string returned is in hex; otherwise, the string is in binary. */
PHP_FUNCTION(cryptopp_get_hmac_key)
{
	zval* resource;
	JHash* hash = NULL;
	zend_bool hex = 0;
	string retval;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|b", &hex))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r|b", &resource, &hex))

	if (hash == NULL) {
		php_error(E_WARNING, "can't get key from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_NON_HMAC(hash->getHashType())) {
		php_error(E_WARNING, "can't get key from regular hash, must be a HMAC hash in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		retval = ((JHMAC*) hash)->getKey(hex ? true : false);
		RETVAL_STRINGL((char*) retval.data(), retval.length(), 1);
	}
}
/* }}} */


/* {{{ proto int cryptopp_set_hmac_key_length(resource hmac, int keylength)
	Set the length of the key. Returns the true keylength on success, false on error. Depending
	on the algorithm being used, the keylength you enter may not necessarily be the keylength that is
	set. */
PHP_FUNCTION(cryptopp_set_hmac_key_length)
{
	zval* resource;
	JHash* hash = NULL;
	unsigned int keylength, actual;
	CRYPTOPP_HASH_OBJ_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &keylength))

	CRYPTOPP_HASH_RSRC_PARAMS(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", &resource, &keylength))

	if (hash == NULL) {
		php_error(E_WARNING, "can't set keylength on null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_NON_HMAC(hash->getHashType())) {
		php_error(E_WARNING, "can't set keylength of regular hash, must be HMAC hash in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		actual = ((JHMAC*) hash)->setKeylength(keylength);

		if (actual != keylength) {
			php_error(E_WARNING, "%s() set keylength to %d, but the requested length was %d", get_active_function_name(TSRMLS_C), actual, keylength);
		}
		RETVAL_LONG(actual);
	}
}
/* }}} */


/* {{{ proto int cryptopp_get_hmac_key_length(resource cipher)
	Returns the length of the secret key. The length returned is for the key in BINARY, not hex. To figure
	out the hex, obviously just multiply the result by two. Returns false on error. */
PHP_FUNCTION(cryptopp_get_hmac_key_length)
{
	zval* resource;
	JHash* hash = NULL;
	CRYPTOPP_HASH_OBJ_NO_PARAMS()

	CRYPTOPP_HASH_RSRC_NO_PARAMS()

	if (hash == NULL) {
		php_error(E_WARNING, "can't get keylength from null resource in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else if (IS_NON_HMAC(hash->getHashType())) {
		php_error(E_WARNING, "can't get keylength from regular hash, must be HMAC hash in %s()", get_active_function_name(TSRMLS_C));
		RETURN_FALSE;
	}
	else {
		RETVAL_LONG(((JHMAC*) hash)->getKeylength());
	}
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: sw=4 ts=4 tw=78 fdm=marker
 * vim<600: sw=4 ts=4 tw=78
 */
