/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: jcipher.cpp,v 1.9 2005/01/06 18:16:42 jay Exp $
*/

#include "jcipher.h"

JCipher::JCipher()
{
	itsMode = ECB_MODE;
	itsPadding = ZEROS_PADDING;
	itsRNG = DEFAULT_RNG;
}

string JCipher::getModeName() const
{
	return getModeName(itsMode);
}

string JCipher::getModeName(const enum Mode mode)
{
	switch (mode)
	{
		case ECB_MODE:
			return "ECB";
		case CBC_MODE:
			return "CBC";
		case CBC_CTS_MODE:
			return "CBC with CTS";
		case CFB_MODE:
			return "CFB";
		case CTR_MODE:
			return "CTR";
		case OFB_MODE:
			return "OFB";
	}

	return "Unknown";
}

enum Mode JCipher::getMode() const
{
	return itsMode;
}

void JCipher::setMode(const enum Mode mode)
{
	itsMode = mode;
	itsPadding = DEFAULT_PADDING;
}

string JCipher::getPaddingName() const
{
	if (itsPadding == DEFAULT_PADDING)
	{
		if (itsMode == ECB_MODE || itsMode == CBC_MODE)
			return "Default cipher padding (PKCS)";
		else
			return "Default cipher padding (none)";
	}
	else
		return getPaddingName(itsPadding);
}

string JCipher::getPaddingName(const enum Padding padding)
{
	switch (padding)
	{
		case NO_PADDING:
			return "None";
		case ZEROS_PADDING:
			return "Zeroes";
		case PKCS_PADDING:
			return "PKCS";
		case ONE_AND_ZEROS_PADDING:
			return "One and zeroes";
		case DEFAULT_PADDING:
			return "Default cipher padding";
	}

	return "Unknown";
}

enum Padding JCipher::getPadding() const
{
	return itsPadding;
}

enum Padding JCipher::setPadding(const enum Padding padding)
{
	if (padding == NO_PADDING && (itsMode == ECB_MODE || itsMode == CBC_MODE))
		return itsPadding;
	else if ((padding == PKCS_PADDING || padding == ONE_AND_ZEROS_PADDING) && (itsMode == CBC_CTS_MODE || itsMode == CTR_MODE || itsMode == OFB_MODE || itsMode == CFB_MODE))
		return itsPadding;
	else
	{
		itsPadding = padding;
		return itsPadding;
	}
}


string JCipher::getRNGName() const
{
	return getRNGName(itsRNG);
}

string JCipher::getRNGName(const enum RNG rng)
{
	switch (rng)
	{
		#ifdef NONBLOCKING_RNG_AVAILABLE	
		case NONBLOCKING_RNG:
			#if defined(CRYPTOPP_WIN32_AVAILABLE) && defined(USE_MS_CRYPTOAPI)
			return "Non-blocking (Microsoft CryptoAPI)";
			#else
			return "Non-blocking (/dev/urandom, etc.)";
			#endif
		#endif

		#ifdef BLOCKING_RNG_AVAILABLE
		case BLOCKING_RNG:
			return "Blocking (/dev/random, etc.)";
		#endif
		
		case RAND_RNG:
			return "System rand() function";
	}

	return "Unknown";
}

enum RNG JCipher::getRNG() const
{
	return itsRNG;
}

enum RNG JCipher::setRNG(const enum RNG rng)
{
	#ifdef NONBLOCKING_RNG_AVAILABLE
	if (rng == NONBLOCKING_RNG)
		itsRNG = rng;
	#endif

	#ifdef BLOCKING_RNG_AVAILABLE
	if (rng == BLOCKING_RNG)
		itsRNG = rng;
	#endif

	if (rng == RAND_RNG)
		itsRNG = rng;
                    
	return itsRNG;
}


unsigned int JCipher::getRounds() const
{
	return itsRounds;
}

unsigned int JCipher::setRounds(const unsigned int rounds)
{
	itsRounds = getValidRounds(rounds);

	return itsRounds;
}

string JCipher::getIV(bool hex) const
{
	if (hex)
		return bin2hex(itsIV);
	else
		return itsIV;
}

void JCipher::setIV(string iv, bool hex)
{
	if (hex)
		itsIV = hex2bin(iv);
	else
		itsIV = iv;
}

void JCipher::setRandIV(const unsigned int size)
{
	itsIV = generateIV(size, itsRNG);
}
