/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: jhash_t.cpp,v 1.7 2005/01/06 18:16:42 jay Exp $
*/

template <typename HASH, enum Hash TYPE>
JHash_Template<HASH, TYPE>::JHash_Template(string plaintext) : JHash(plaintext)
{
	itsHashModule = new HASH;
}

template <typename HASH, enum Hash TYPE>
Hash JHash_Template<HASH, TYPE>::getHashType() const
{
	return TYPE;
}

template <typename HASH, enum Hash TYPE>
bool JHash_Template<HASH, TYPE>::hash()
{
	itsHashtext.erase();
    
	StringSource s(itsPlaintext, true, new HashFilter(*itsHashModule, new HexEncoder(new StringSink(itsHashtext))));
	return true;
}

template <typename HASH, enum Hash TYPE>
bool JHash_Template<HASH, TYPE>::validate()
{
	return validate(itsPlaintext, itsHashtext);
}

template <typename HASH, enum Hash TYPE>
bool JHash_Template<HASH, TYPE>::validate(string plaintext, string hashtext)
{
	if (itsHashModule == NULL)
		throw;

	return itsHashModule->VerifyDigest((const byte*) hex2bin(hashtext).data(), (const byte*) plaintext.data(), plaintext.length());
}

template <typename HASH, enum Hash TYPE>
string JHash_Template<HASH, TYPE>::hashPHPStream(php_stream* in)
{
	if (itsHashModule == NULL)
		throw;

	string retval;
	try
	{
		PHPStreamSource f(&in, true, new HashFilter(*itsHashModule, new HexEncoder(new StringSink(retval))));
	}
	catch (Exception e)
	{
		throw e;
	}
	return retval;
} 


/* This is deprecated. It was used before using php_streams. Use it
   if you're using this code in something other than the cryptopp PHP
   extension...
template <typename HASH, enum Hash TYPE>
string JHash_Template<HASH, TYPE>::hashFile(const string filename)
{
	if (itsHashModule == NULL)
		throw;

	string retval;
	try
	{
		FileSource f(filename.c_str(), true, new HashFilter(*itsHashModule, new HexEncoder(new StringSink(retval))));
	}
	catch (FileStore::OpenErr e)
	{
		throw e;
	}
	return retval;
} */
