/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: jhelpers.cpp,v 1.15 2005/01/06 18:16:42 jay Exp $
*/

#include "jhelpers.h"

using namespace CryptoPP;

string bin2hex(string bin)
{
	string retval;
	StringSource(bin, true, new HexEncoder(new StringSink(retval)));
	return retval;
}

string hex2bin(string hex)
{
	string retval;
	StringSource(hex, true, new HexDecoder(new StringSink(retval)));
	return retval;
}

string strtoupper(string str)
{
	for (unsigned int i = 0; i < str.length(); i++)
		str[i] = toupper(str[i]);
	return str;
}

string generateIV(const unsigned int size, const enum RNG rng)
{
	string retval;
	RandomNumberGenerator* randPool = NULL;
	TSRMLS_FETCH();

	#ifdef NONBLOCKING_RNG_AVAILABLE
	if (rng == NONBLOCKING_RNG)
		randPool = new NonblockingRng;
	#endif

	#if defined(NONBLOCKING_RNG_AVAILABLE) && defined(BLOCKING_RNG_AVAILABLE)
	else
	#endif

	#ifdef BLOCKING_RNG_AVAILABLE
	if (rng == BLOCKING_RNG)
		randPool = new BlockingRng;
	#endif

	if (rng == RAND_RNG)
	{
		for (unsigned int i = 0; i < size; i++)
			retval += (char)(255.0 * php_rand(TSRMLS_C) / RAND_MAX);
	}
	else if (randPool != NULL)
	{
		for (unsigned int i = 0; i < size; i++)
			retval += (char) randPool->GenerateByte();
		delete randPool;
	}

	return retval;
}

unsigned int checkBounds(unsigned int length, unsigned int min, unsigned int max, unsigned short int multiple)
{
	if (min == max)
		return min;
	else if (length < min)
		return min;
	else if (length > max)
		return max;
	else
		return (length + multiple - 1) - ((length + multiple - 1) % multiple);
}
