/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: jsink.cpp,v 1.4 2005/01/06 18:16:43 jay Exp $
*/

#include "jsink.h"

void PHPStreamStore::StoreInitialize(const NameValuePairs& parameters)
{
	TSRMLS_FETCH();
	const char* fileName;
	if (parameters.GetValue("InputFileName", fileName))
	{
		m_stream = php_stream_fopen(fileName, (parameters.GetValueWithDefault("InputBinaryMode", true) ? "rb" : "r"), NULL);
		if (!m_stream)
		{
			m_stream = NULL;
			throw OpenErr(fileName);
		}
	}
	else
	{
		m_stream = NULL;
		parameters.GetValue("InputStreamPointer", m_stream);
	}

	m_waiting = false;
}

unsigned int PHPStreamStore::Peek(byte& outByte) const
{
	TSRMLS_FETCH();
	if (!m_stream || php_stream_eof(m_stream))
		return 0;
	else
		return 1;
}

unsigned int PHPStreamStore::TransferTo2(BufferedTransformation& target, unsigned long& transferBytes, const std::string& channel, bool blocking)
{
	TSRMLS_FETCH();
	if (!m_stream)
	{
		transferBytes = 0;
		return 0;
	}

	unsigned long size = transferBytes;
	transferBytes = 0;

	if (m_waiting)
		goto output;

	while (size && !php_stream_eof(m_stream))
	{
		{
			unsigned int spaceSize = 1024;
			m_space = HelpCreatePutSpace(target, channel, 1, (unsigned int) STDMIN(size, (unsigned long) UINT_MAX), spaceSize);

			m_len = php_stream_read(m_stream, (char*) m_space, STDMIN(size, (unsigned long) spaceSize));
		}
		unsigned int blockedBytes;
		output:
		blockedBytes = target.ChannelPutModifiable2(channel, m_space, m_len, 0, blocking);
		m_waiting = blockedBytes > 0;
		if (m_waiting)
			return blockedBytes;
		size -= m_len;
		transferBytes += m_len;
	}
	if (!php_stream_eof(m_stream))
		throw ReadErr();
	return 0;
}

void PHPStreamSink::IsolatedInitialize(const NameValuePairs& parameters)
{
	TSRMLS_FETCH();
	const char* fileName;
	if (parameters.GetValue("OutputFileName", fileName))
	{
		m_stream = php_stream_fopen(fileName, (parameters.GetValueWithDefault("OutputBinaryMode", true) ? "wb" : "w"), NULL);
		if (!m_stream)
		{
			m_stream = NULL;
			throw OpenErr(fileName);
		}
	}
	else
	{
		m_stream = NULL;
		parameters.GetValue("OutputStreamPointer", m_stream);
	}
}

unsigned int PHPStreamSink::Put2(const byte* inString, unsigned int length, int messageEnd, bool blocking)
{
	TSRMLS_FETCH();
	if (!m_stream)
		throw Err("FileSink: output stream not opened");

	php_stream_write(m_stream, (const char*) inString, length);

	if (messageEnd)
	{
		if (php_stream_flush(m_stream) != 0)
			throw WriteErr();
	}

	return 0;
}

