/*
   +----------------------------------------------------------------------+
   | Copyright (c) 2002-2005 J Smith <dark_panda@hushmail.com>            |
   | Crypto++ sources (not included) copyright (c) 1995-2003 Wei Dai      |
   +----------------------------------------------------------------------+
   | Check out COPYING for the exact license.                             |
   +----------------------------------------------------------------------+

$Id: jstream_t.cpp,v 1.7 2005/01/06 18:16:43 jay Exp $
*/

template <typename INFO, enum Cipher TYPE>
JStream_Template<INFO, TYPE>::JStream_Template()
{
	this->itsKeylength = INFO::DEFAULT_KEYLENGTH;
}

template <typename INFO, enum Cipher TYPE>
Cipher JStream_Template<INFO, TYPE>::getCipherType() const
{
	return TYPE;
}

template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::encrypt()
{
	StreamTransformation* cipher = NULL;

	cipher = getEncryptionObject();

	if (cipher != NULL)
	{
		this->itsCiphertext.erase();
		StringSource(this->itsPlaintext, true, new StreamTransformationFilter(*cipher, new StringSink(this->itsCiphertext)));
		delete cipher;
		return true;
	}
	else
		return false;
}

template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::decrypt()
{
	StreamTransformation* cipher = NULL;

	cipher = getDecryptionObject();

	if (cipher != NULL)
	{
		this->itsPlaintext.erase();
		StringSource(this->itsCiphertext, true, new StreamTransformationFilter(*cipher, new StringSink(this->itsPlaintext)));
		delete cipher;
		return true;
	}
	else
		return false;
}

template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::encryptPHPStream(php_stream* in, php_stream* out)
{
	StreamTransformation* cipher = NULL;

	cipher = getEncryptionObject();

	if (cipher != NULL)
	{
		try
		{
			PHPStreamSource(&in, true, new StreamTransformationFilter(*cipher, new PHPStreamSink(&out)));
		}
		catch (PHPStreamStore::OpenErr e)
		{
			delete cipher;
			throw e;
		}
		delete cipher;
	}

	return true;
}

template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::decryptPHPStream(php_stream* in, php_stream* out)
{
	StreamTransformation* cipher = NULL;

	cipher = getDecryptionObject();

	if (cipher != NULL)
	{
		try
		{
			PHPStreamSource(&in, true, new StreamTransformationFilter(*cipher, new PHPStreamSink(&out)));
		}
		catch (PHPStreamStore::OpenErr e)
		{
			delete cipher;
			throw e;
		}
		delete cipher;
	}

	return true;
}

/* These are deprecated. They were used before using php_streams. Use them
   if you're using this code in something other than the cryptopp PHP
   extension...
template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::encryptFile(const string in, const string out)
{
	StreamTransformation* cipher = NULL;

	cipher = getEncryptionObject();

	if (cipher != NULL)
	{
		try
		{
			FileSource(in.c_str(), true, new StreamTransformationFilter(*cipher, new FileSink(out.c_str())));
		}
		catch (FileStore::OpenErr e)
		{
			delete cipher;
			throw e;
		}
		delete cipher;
	}

	return true;
}

template <typename INFO, enum Cipher TYPE>
bool JStream_Template<INFO, TYPE>::decryptFile(const string in, const string out)
{
	StreamTransformation* cipher = NULL;

	cipher = getDecryptionObject();

	if (cipher != NULL)
	{
		try
		{
			FileSource(in.c_str(), true, new StreamTransformationFilter(*cipher, new FileSink(out.c_str())));
		}
		catch (FileStore::OpenErr e)
		{
			delete cipher;
			throw e;
		}
		delete cipher;
	}

	return true;
} */

