<?php

define('USE_ZE2_CLASSES', true);

if (USE_ZE2_CLASSES && version_compare(zend_version(), '2.0.0-dev', '>='))
{
	print "Using ZE2 objects\n\n";
	function runtest($args)
	{
		global $passedtests, $failedtests;

		$cipher = new cryptoppCipher($args['ALGORITHM']);
		$args['PLAINTEXT'] = strtoupper($args['PLAINTEXT']);

		if (isset($args['MODE']))
			$cipher->setMode($args['MODE']);

		if (isset($args['PAD']))
			$cipher->setPadding($args['PAD']);
		else if ($args['ALGORITHM'] > CRYPTOPP_STREAM_SEAL)
			$cipher->setPadding(CRYPTOPP_PAD_ZEROES);

		if (isset($args['IV']))
			$cipher->setIV($args['IV'], (isset($args['IV_HEX']) ? $args['IV_HEX'] : true));

		if (isset($args['KEY']))
			$cipher->setKey($args['KEY'], (isset($args['KEY_HEX']) ? $args['KEY_HEX'] : true));

		if (isset($args['KEYLEN']))
			$cipher->setKeyLength($args['KEYLEN']);

		if (isset($args['ROUNDS']))
			$cipher->setRounds($args['ROUNDS']);

		if (isset($args['EFF_KEYLEN']))
			$cipher->setEffectiveKeyLength($args['EFF_KEYLEN']);

		$cipher->setPlaintext($args['PLAINTEXT'], (isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true));

		$cipher->encrypt();

		$cipher->setPlaintext('');

		$cipher->decrypt();

		if ($cipher->getCiphertext() != $args['EXPECTED'] || $cipher->getPlaintext((isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true)) != $args['PLAINTEXT']) {
			print "failed:";
			++$failedtests;
		}
		else {
			print "passed:";
			++$passedtests;
		}

		print "\t" . $cipher->getKey(true) . "\t" . $cipher->getPlaintext(true) . "\t" . $cipher->getCiphertext() . "\n";
		if ($cipher->getCiphertext() != $args['EXPECTED'])
			print "Expected ciphertext: " . $args['EXPECTED'] . "\n";
		if ($cipher->getPlaintext((isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true)) != $args['PLAINTEXT'])
			print "Expected plaintext: " . $args['PLAINTEXT'] . "\n";
		unset($cipher);
	}

	function runhashtest($algorithm, $plaintext, $expected, $key = null, $bin = false)
	{
		global $passedtests, $failedtests;

		$hash = new cryptoppHash($algorithm);
		$hash->setPlaintext($plaintext);

		if ($key !== null)
			$hash->setHMACKey($key, true);

		$hash->hash();

		if ($hash->getHash() == $expected) {
			print "passed:";
			++$passedtests;
		}
		else {
			print "failed:";
			++$failedtests;
		}

		print "\t";
        if ($bin)
            print bin2hex($hash->getPlaintext());
        else
            print $hash->getPlaintext();
        print "\t" . $hash->getHash() . "\n";

        if ($hash->getHash() != $expected)
            print "Expected hash:\t" . $expected . "\n";
		unset($hash);
	}
}
else
{
	print "Not using ZE2 objects\n\n";
	function runtest($args)
	{
		global $passedtests, $failedtests;

		$cipher = cryptopp_create_cipher($args['ALGORITHM']);
		$args['PLAINTEXT'] = strtoupper($args['PLAINTEXT']);

		if (isset($args['MODE']))
			cryptopp_set_mode($cipher, $args['MODE']);

		if (isset($args['PAD']))
			cryptopp_set_padding($cipher, $args['PAD']);
		else if ($args['ALGORITHM'] > CRYPTOPP_STREAM_SEAL)
			cryptopp_set_padding($cipher, CRYPTOPP_PAD_ZEROES);

		if (isset($args['IV']))
			cryptopp_set_iv($cipher, $args['IV'], (isset($args['IV_HEX']) ? $args['IV_HEX'] : true));

		if (isset($args['KEY']))
			cryptopp_set_key($cipher, $args['KEY'], (isset($args['KEY_HEX']) ? $args['KEY_HEX'] : true));

		if (isset($args['KEYLEN']))
			cryptopp_set_key_length($cipher, $args['KEYLEN']);

		if (isset($args['ROUNDS']))
			cryptopp_set_rounds($cipher, $args['ROUNDS']);

		if (isset($args['EFF_KEYLEN']))
			cryptopp_set_effective_key_length($cipher, $args['EFF_KEYLEN']);

		cryptopp_set_plaintext($cipher, $args['PLAINTEXT'], (isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true));

		cryptopp_encrypt($cipher);

		cryptopp_set_plaintext($cipher, '');

		cryptopp_decrypt($cipher);

		if (cryptopp_get_ciphertext($cipher) != $args['EXPECTED'] || cryptopp_get_plaintext($cipher, (isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true)) != $args['PLAINTEXT']) {
			print "failed:";
			++$failedtests;
		}
		else {
			print "passed:";
			++$passedtests;
		}

		print "\t" . cryptopp_get_key($cipher, true) . "\t" . cryptopp_get_plaintext($cipher, true) . "\t" . cryptopp_get_ciphertext($cipher) . "\n";
		if (cryptopp_get_ciphertext($cipher) != $args['EXPECTED'])
			print "Expected ciphertext: " . $args['EXPECTED'] . "\n";
		if (cryptopp_get_plaintext($cipher, (isset($args['PLAINTEXT_HEX']) ? $args['PLAINTEXT_HEX'] : true)) != $args['PLAINTEXT'])
			print "Expected plaintext: " . $args['PLAINTEXT'] . "\n";
		cryptopp_destroy_cipher($cipher);
	}



	function runhashtest($algorithm, $plaintext, $expected, $key = null, $bin = false)
	{
		global $passedtests, $failedtests;

		$hash = cryptopp_create_hash($algorithm);
		cryptopp_set_hash_plaintext($hash, $plaintext);

		if ($key !== null)
			cryptopp_set_hmac_key($hash, $key, true);

		cryptopp_hash($hash);

		if (cryptopp_get_hashtext($hash) == $expected) {
			print "passed:";
			++$passedtests;
		}
		else {
			print "failed:";
			++$failedtests;
		}

		print "\t";
        if ($bin) 
            print bin2hex(cryptopp_get_hash_plaintext($hash));
        else
            print cryptopp_get_hash_plaintext($hash);
        print "\t" . cryptopp_get_hashtext($hash) . "\n";

        if (cryptopp_get_hashtext($hash) != $expected)
            print "Expected:\t" . $expected . "\n";

		cryptopp_destroy_hash($hash);
	}
}

?>
