<?php

//error_reporting(E_ALL);

if (!extension_loaded('cryptopp'))
	die("The cryptopp extension hasn't been loaded!\n\n");

print "\nPerforming Crypto++ tests:\n\n";

require('tests.php');

function line()
{
	 print "\n----------------------------------------------------------------------\n\n";
}


$failedtests = 0;
$passedtests = 0;


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_3WAY))
{
	print "3Way tests:\n";
	$file = file('3wayval.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_3WAY,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2])),
					));
	}
}
else
	print "Skipping 3way tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_AES))
{
	print "AES (Rijndael) tests:\n";
	$file = file('rijndael.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_AES,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
}
else
	print "Skipping AES (Rijndael) tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_BLOWFISH))
{
	print "Blowfish tests:\n";
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_BLOWFISH,
					'KEY' => 'abcdefghijklmnopqrstuvwxyz',
					'KEY_HEX' => false,
					'PLAINTEXT' => 'BLOWFISH',
					'PLAINTEXT_HEX' => false,
					'EXPECTED' => '324ED0FEF413A203'
				));
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_BLOWFISH,
					'KEY' => 'Who is John Galt?',
					'KEY_HEX' => false,
					'PLAINTEXT' => 'fedcba9876543210',
					'PLAINTEXT_HEX' => true,
					'EXPECTED' => 'CC91732B8022F684'
				));
}
else
	print "Skipping Blowfish tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_CAMELLIA))
{
    print "Camellia tests:\n";
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_CAMELLIA,
                    'KEY' => '80000000000000000000000000000000',
                    'PLAINTEXT' => '00000000000000000000000000000000',
                    'EXPECTED' => '6C227F749319A3AA7DA235A9BBA05A2C'
                ));
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_CAMELLIA,
                    'KEY' => '00000000000000000000000000000000',
                    'PLAINTEXT' => '80000000000000000000000000000000',
                    'EXPECTED' => '07923A39EB0A817D1C4D87BDB82D1F1C'
                ));
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_CAMELLIA,
                    'KEY' => '00000000000000000000000000000000',
                    'PLAINTEXT' => '00000000000000000000000000000000',
                    'EXPECTED' => '3D028025B156327C17F762C1F2CBCA71'
                ));
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_CAMELLIA,
                    'KEY' => 'A5A5A5A5A5A5A5A5A5A5A5A5A5A5A5A5',
                    'PLAINTEXT' => 'A5A5A5A5A5A5A5A5A5A5A5A5A5A5A5A5',
                    'EXPECTED' => '1F55093C234648C5C9C781E8C9FD42C4'
                ));
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_CAMELLIA,
                    'KEY' => 'FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE',
                    'PLAINTEXT' => 'FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE',
                    'EXPECTED' => 'ACDBB0F5A00E3CF63A89D9C09B44A058'
                ));
}
else
    print "Skipping Camellia tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_CAST128))
{
	print "CAST-128 tests:\n";
	$file = file('cast128v.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_CAST128,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping CAST-128 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_CAST256))
{
	print "CAST-256 tests:\n";
	$file = file('cast256v.dat');
	foreach ($file as $key => $var)
	{
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_CAST256,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping CAST-256 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES))
{
	print "DES tests:\n";
	$file = file('descert.dat');
	foreach ($file as $key => $var)
	{
		$blocks = explode(' ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}

	$key = '0123456789abcdef';
	$iv = '1234567890abcdef';
	$plaintext = '4e6f77206973207468652074696d6520666f7220616c6c20';

	$expected = 'E5C7CDDE872BF27C43E934008C389C0F683788499A7C05F6';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_CBC,
					'PAD' => CRYPTOPP_PAD_ZEROES
				));

	$expected = 'E5C7CDDE872BF27C43E934008C389C0F683788499A7C05F662C16A27E4FCF277';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_CBC,
					'PAD' => CRYPTOPP_PAD_PKCS
				));

	$expected = 'E5C7CDDE872BF27C683788499A7C05F643E934008C389C0F';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_CBC_CTS
				));

	$expected = 'F3096249C7F46E51A69E839B1A92F78403467133898EA622';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_CFB
				));

	$expected = 'F3096249C7F46E5135F24A242EEB3D3F3D6D5BE3255AF8C3';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_OFB
				));

	$expected = 'F3096249C7F46E51163A8CA0FFC94C27FA2F80F480B86F75';
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES,
					'KEY' => $key,
					'PLAINTEXT' => $plaintext,
					'EXPECTED' => $expected,
					'IV' => $iv,
					'IV_HEX' => false,
					'MODE' => CRYPTOPP_MODE_COUNTER
				));
}
else
	print "Skipping DES tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_EDE2) || cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_EDE3) || cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_XEX3))
{
	print "Triple DES tests:\n";
	$file = file("3desval.dat");

	if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_EDE2))
	{
		print "DES EDE2 tests:\n";
		$blocks = explode('  ', $file[0]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES_EDE2,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
	else
		print "\n\nSkipping DES EDE2 tests\n\n";

	if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_EDE3))
	{
		print "EDE3:\n";
		$blocks = explode('  ', $file[1]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES_EDE3,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
		else
			print "\n\nSkpping DES EDE3 tests\n\n";

	if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DES_XEX3))
	{
		print "XEX3:\n";
		$blocks = explode('  ', $file[2]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DES_XEX3,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
	else
		print "\n\nSkipping DES XEX3 tests\n\n";
}
else
	print "Skipping Triple DES tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DIAMOND2))
{
	print "Diamond2 tests:\n";
	$file = file('diamond.dat');
	$blocks = explode('  ', $file[1]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 15
				));

	$blocks = explode('  ', $file[3]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 14
				));

	$blocks = explode('  ', $file[5]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 9
				));
}
else
	print "Skipping Diamond2 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_DIAMOND2_LITE))
{
	print "Diamond2 Lite tests:\n";
	$file = file('diamond.dat');
	$blocks = explode('  ', $file[0]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2_LITE,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 30
				));

	$blocks = explode('  ', $file[2]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2_LITE,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 11
				));

	$blocks = explode('  ', $file[4]);
	runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_DIAMOND2_LITE,
					'KEY' => $blocks[0],
					'PLAINTEXT' => $blocks[1],
					'EXPECTED' => trim($blocks[2]),
					'ROUNDS' => 10
				));
}
else
	print "Skipping Diamond2 Lite tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_GOST))
{
	print "GOST tests:\n";
	$file = file('gostval.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_GOST,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping GOST tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_IDEA))
{
	print "IDEA tests:\n";
	$file = file('ideaval.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_IDEA,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
}
else
	print "Skipping IDEA tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_MARS))
{
	print "MARS tests:\n";
	$file = file('marsval.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('   ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_MARS,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
}
else
	print "Skipping MARS tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_RC2))
{
	print "RC2 tests:\n";
	$file = file('rc2val.dat');
	for ($i = 0; $i < count($file); $i++) {
		$keylen = trim($file[$i++]);
		$effective = trim($file[$i++]);
		$key = trim($file[$i++]);
		$plaintext = trim($file[$i++]);
		$expected = strtoupper(trim($file[$i++]));

		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_RC2,
						'KEY' => $key,
						'PLAINTEXT' => $plaintext,
						'EXPECTED' => $expected,
						'KEYLEN' => (int) $keylen,
						'EFF_KEYLEN' => $effective
					));
	}
}
else
	print "Skipping RC2 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_RC5))
{
	print "RC5 tests:\n";
	$file = file('rc5val.dat');
	foreach ($file as $key => $var)
	{
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_RC5,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2]),
						'ROUNDS' => 12
					));
	}
}
else
	print "Skipping RC5 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_RC6))
{
	print "RC6 tests:\n";
	$file = file('rc6val.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_RC6,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping RC6 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SAFER_K))
{
	print "SAFER-K tests:\n";
	$file = file("saferval.dat");
	for ($i = 0; $i < 4; $i++) {
		$blocks = explode('   ', $file[$i]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SAFER_K,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2])
					));
	}
	for ($i = 4; $i < 8; $i++) {
		$blocks = explode('   ', $file[$i]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SAFER_K,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2]),
						'ROUNDS' => 12
					));
	}
}
else
	print "Skipping SAFER-K tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SAFER_SK))
{
	print "SAFER-SK tests:\n";
	$file = file("saferval.dat");
	for ($i = 8; $i < 12; $i++) {
		$blocks = explode('   ', $file[$i]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SAFER_SK,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2]),
						'ROUNDS' => 6
					));
	}
	for ($i = 12; $i < 16; $i++) {
		$blocks = explode('   ', $file[$i]);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SAFER_SK,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => trim($blocks[2]),
						'ROUNDS' => 10
					));
	}
}
else
	print "Skipping SAFER-SK tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SERPENT))
{
	print "Serpent tests:\n";
	$file = file('serpentv.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SERPENT,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping Serpent tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SHACAL2))
{
    print "SHACAL2 tests:\n";
    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_SHACAL2,
                    'KEY' => '80000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000',
                    'PLAINTEXT' => '0000000000000000000000000000000000000000000000000000000000000000',
                    'EXPECTED' => '361AB6322FA9E7A7BB23818D839E01BDDAFDF47305426EDD297AEDB9F6202BAE'
                ));

    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_SHACAL2,
                    'KEY' => '00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000',
                    'PLAINTEXT' => '8000000000000000000000000000000000000000000000000000000000000000',
                    'EXPECTED' => '2CAE7C0460EE2FC3200923A1B6C2ABEEA746C8B44F6C3FB941BD3AF02A3E6E3E'
                ));

    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_SHACAL2,
                    'KEY' => '00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000',
                    'PLAINTEXT' => '0000000000000000000000000000000000000000000000000000000000000000',
                    'EXPECTED' => '7CA51614425C3BA8CE54DD2FC2020AE7B6E574D198136D0FAE7E26CCBF0BE7A6'
                ));

    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_SHACAL2,
                    'KEY' => '24242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424',
                    'PLAINTEXT' => '2424242424242424242424242424242424242424242424242424242424242424',
                    'EXPECTED' => '9766D3F9E73987EE10A20605C8338C0759A7024CF2DF549DE5084EC4902C550A'
                ));

    runtest(array(  'ALGORITHM' => CRYPTOPP_CIPHER_SHACAL2,
                    'KEY' => 'FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE',
                    'PLAINTEXT' => 'FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE',
                    'EXPECTED' => '5DC15C83E7C4F5AAF8D8482ED1E443271B28B59288783DBCDDEC3544E3368A6E'
                ));
}
else
    print "Skipping SHACAL2 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SHARK))
{
	print "SHARK tests:\n";
	$file = file('sharkval.dat');
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SHARK,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping SHARK tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SKIPJACK))
{
	print "SKIPJACK tests:\n";
	$file = file("skipjack.dat");
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SKIPJACK,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping SKIPJACK tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_SQUARE))
{
	print "Square tests:\n";
	$file = file("squareva.dat");
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_SQUARE,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping SQUARE tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_CIPHER_TWOFISH))
{
	print "Twofish tests:\n";
	$file = file("twofishv.dat");
	foreach ($file as $key => $var) {
		$blocks = explode('  ', $var);
		runtest(array(	'ALGORITHM' => CRYPTOPP_CIPHER_TWOFISH,
						'KEY' => $blocks[0],
						'PLAINTEXT' => $blocks[1],
						'EXPECTED' => strtoupper(trim($blocks[2]))
					));
	}
}
else
	print "Skipping Twofish tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_STREAM_ARC4))
{
	print "ARC4 tests:\n";

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_ARC4,
					'KEY' => '0123456789abcdef',
					'PLAINTEXT' => '0123456789abcdef',
					'EXPECTED' => '75B7878099E0C596'
				));

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_ARC4,
					'KEY' => '0123456789abcdef',
					'PLAINTEXT' => '0000000000000000',
					'EXPECTED' => '7494C2E7104B0879'
				));

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_ARC4,
					'KEY' => '0000000000000000',
					'PLAINTEXT' => '0000000000000000',
					'EXPECTED' => 'DE188941A3375D3A'
				));

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_ARC4,
					'KEY' => 'ef012345',
					'PLAINTEXT' => '00000000000000000000',
					'EXPECTED' => 'D6A141A7EC3C38DFBD61'
				));

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_ARC4,
					'KEY' => '0123456789ABCDEF',
					'PLAINTEXT' => str_repeat('01', 512),
					'EXPECTED' => 	'7595C3E6114A09780C4AD452338E1FFD9A1BE949' .
									'8F813D76533449B6778DCAD8C78A8D2BA9AC6608' .
									'5D0E53D59C26C2D1C490C1EBBE0CE66D1B6B1B13' .
									'B6B919B847C25A91447A95E75E4EF16779CDE8BF' .
									'0A95850E32AF9689444FD377108F98FDCBD4E726' .
									'567500990BCC7E0CA3C4AAA304A387D20F3B8FBB' .
									'CD42A1BD311D7A4303DDA5AB078896AE80C18B0A' .
									'F66DFF319616EB784E495AD2CE90D7F772A81747' .
									'B65F62093B1E0DB9E5BA532FAFEC47508323E671' .
									'327DF9444432CB7367CEC82F5D44C0D00B67D650' .
									'A075CD4B70DEDD77EB9B10231B6B5B741347396D' .
									'62897421D43DF9B42E446E358E9C11A9B2184ECB' .
									'EF0CD8E7A877EF968F1390EC9B3D35A5585CB009' .
									'290E2FCDE7B5EC66D9084BE44055A619D9DD7FC3' .
									'166F9487F7CB272912426445998514C15D53A18C' .
									'864CE3A2B7555793988126520EACF2E3066E230C' .
									'91BEE4DD5304F5FD0405B35BD99C73135D3D9BC3' .
									'35EE049EF69B3867BF2D7BD1EAA595D8BFC0066F' .
									'F8D31509EB0C6CAA006C807A623EF84C3D33C195' .
									'D23EE320C40DE0558157C822D4B8C569D849AED5' .
									'9D4E0FD7F379586B4B7FF684ED6A189F7486D49B' .
									'9C4BAD9BA24B96ABF924372C8A8FFFB10D553549' .
									'00A77A3DB5F205E1B99FCD8660863A159AD4ABE4' .
									'0FA48934163DDDE542A6585540FD683CBFD8C00F' .
									'12129A284DEACC4CDEFE58BE7137541C047126C8' .
									'D49E2755AB181AB7E940B0C0'
				));
}
else
	print "Skipping ARC4 tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_STREAM_PANAMA_LITTLE_ENDIAN))
{
	print "Panama cipher (little endian) tests:\n";
	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_PANAMA_LITTLE_ENDIAN,
				'KEY' => '000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f',
				'PLAINTEXT' => '000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f',
				'EXPECTED' => 'F07F5FF2CCD01A0A7D44ACD6D239C2AF0DA1FF35275BAF5DFA6E09411B79D8B9'
			));
}
else
	print "Skipping Panama cipher (little endian) tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_STREAM_PANAMA_BIG_ENDIAN))
{
	print "Panama cipher (big endian) tests:\n";
	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_PANAMA_BIG_ENDIAN,
				'KEY' => '000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f',
				'PLAINTEXT' => '000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f',
				'EXPECTED' => 'E12E2F6BA41AE832D888DA9FA6863BC37C0E996F190A1711330322D37BD98CA4'
			));
}
else
	print "Skipping Panama cipher (big endian) tests";

line();


if (cryptopp_cipher_enabled(CRYPTOPP_STREAM_SEAL))
{
	print "SEAL tests:\n";

	runtest(array(	'ALGORITHM' => CRYPTOPP_STREAM_SEAL,
				'KEY' => '67452301efcdab8998badcfe10325476c3d2e1f0',
				'PLAINTEXT' => '37a005959b84c49ca4be1e050673530f5fb097fdf6a13fbd6c2cdecd81fdee7c',
				'EXPECTED' => 'DF2F1CCE656F6B0AA8F3663DF53FF26330F27FCBCC0E220B63AA483BB7343BD3'
			));
}
else
	print "Skipping SEAL tests";

line();


print "\n\nStarting hash tests:\n\n";


if (cryptopp_hash_enabled(CRYPTOPP_HASH_HAVAL))
{
	print "HAVAL tests:\n";

	// skip the first four until we have digest size support

	//runhashtest(CRYPTOPP_HASH_HAVAL3, "", strtoupper(bin2hex("C68F39913F901F3DDF44C707357A7D70")));
	//runhashtest(CRYPTOPP_HASH_HAVAL3, "a", strtoupper(bin2hex("4DA08F514A7275DBC4CECE4A347385983983A830")));
	//runhashtest(CRYPTOPP_HASH_HAVAL4, "HAVAL", strtoupper(bin2hex("0C1396D7772689C46773F3DAACA4EFA982ADBFB2F1467EEA")));
	//runhashtest(CRYPTOPP_HASH_HAVAL4, "0123456789", strtoupper(bin2hex("BEBD7816F09BAEECF8903B1B9BC672D9FA428E462BA699F814841529")));
	runhashtest(CRYPTOPP_HASH_HAVAL5, "abcdefghijklmnopqrstuvwxyz", "C9C7D8AFA159FD9E965CB83FF5EE6F58AEDA352C0EFF005548153A61551C38EE");
	runhashtest(CRYPTOPP_HASH_HAVAL5, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "B45CB6E62F2B1320E4F8F1B0B273D45ADD47C321FD23999DCF403AC37636D963");
}
else
	print "Skipping HAVAL tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_MD2))
{
	print "MD2 tests:\n";

	runhashtest(CRYPTOPP_HASH_MD2, "", strtoupper("8350e5a3e24c153df2275c9f80692773"));
	runhashtest(CRYPTOPP_HASH_MD2, "a", strtoupper("32ec01ec4a6dac72c0ab96fb34c0b5d1"));
	runhashtest(CRYPTOPP_HASH_MD2, "abc", strtoupper("da853b0d3f88d99b30283a69e6ded6bb"));
	runhashtest(CRYPTOPP_HASH_MD2, "message digest", strtoupper("ab4f496bfb2a530b219ff33031fe06b0"));
	runhashtest(CRYPTOPP_HASH_MD2, "abcdefghijklmnopqrstuvwxyz", strtoupper("4e8ddff3650292ab5a4108c3aa47940b"));
	runhashtest(CRYPTOPP_HASH_MD2, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper("da33def2a42df13975352846c30338cd"));
	runhashtest(CRYPTOPP_HASH_MD2, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper("d5976f79d83d3a0dc9806c3c66f3efd8"));
}
else
	print "Skipping MD2 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_MD4))
{
	print "MD4 tests:\n";

	runhashtest(CRYPTOPP_HASH_MD4, "", "31D6CFE0D16AE931B73C59D7E0C089C0");
	runhashtest(CRYPTOPP_HASH_MD4, "a", "BDE52CB31DE33E46245E05FBDBD6FB24");
	runhashtest(CRYPTOPP_HASH_MD4, "abc", "A448017AAF21D8525FC10AE87AA6729D");
	runhashtest(CRYPTOPP_HASH_MD4, "message digest", "D9130A8164549FE818874806E1C7014B");
	runhashtest(CRYPTOPP_HASH_MD4, "abcdefghijklmnopqrstuvwxyz", "D79E1C308AA5BBCDEEA8ED63DF412DA9");
	runhashtest(CRYPTOPP_HASH_MD4, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "043F8582F241DB351CE627E153E7F0E4");
	runhashtest(CRYPTOPP_HASH_MD4, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", "E33B4DDC9C38F2199C3E7B164FCC0536");
}
else
	print "Skipping MD4 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_MD5))
{
	print "MD5 tests:\n";

	runhashtest(CRYPTOPP_HASH_MD5, "", strtoupper(md5("")));
	runhashtest(CRYPTOPP_HASH_MD5, "a", strtoupper(md5("a")));
	runhashtest(CRYPTOPP_HASH_MD5, "abc", strtoupper(md5("abc")));
	runhashtest(CRYPTOPP_HASH_MD5, "message digest", strtoupper(md5("message digest")));
	runhashtest(CRYPTOPP_HASH_MD5, "abcdefghijklmnopqrstuvwxyz", strtoupper(md5("abcdefghijklmnopqrstuvwxyz")));
	runhashtest(CRYPTOPP_HASH_MD5, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper(md5("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")));
	runhashtest(CRYPTOPP_HASH_MD5, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper(md5("12345678901234567890123456789012345678901234567890123456789012345678901234567890")));
}
else
	print "Skipping MD5 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_PANAMA))
{
	print "Panama tests:\n\n";

	print "Little endian:\n";
	runhashtest(CRYPTOPP_HASH_PANAMA_LITTLE_ENDIAN, "", strtoupper("aa0cc954d757d7ac7779ca3342334ca471abd47d5952ac91ed837ecd5b16922b"));
	runhashtest(CRYPTOPP_HASH_PANAMA_LITTLE_ENDIAN, "The quick brown fox jumps over the lazy dog", strtoupper("5f5ca355b90ac622b0aa7e654ef5f27e9e75111415b48b8afe3add1c6b89cba1"));

	print "\n\nBig endian:\n";
	runhashtest(CRYPTOPP_HASH_PANAMA_BIG_ENDIAN, "", strtoupper("e81aa04523532dd7267e5c5bc3ba0e289837a62ba032350351980e960a84b0af"));
	runhashtest(CRYPTOPP_HASH_PANAMA_BIG_ENDIAN, "The quick brown fox jumps over the lazy dog", strtoupper("8fa7dadce0110f979a0b795e76b2c25628d8bda88747758149c42e3bc13f85bc"));
}
else
	print "Skipping Panama hash tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_RIPEMD128))
{
    print "RIPEMD128 tests:\n";

    runhashtest(CRYPTOPP_HASH_RIPEMD128, "", strtoupper("cdf26213a150dc3ecb610f18f6b38b46"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "a", strtoupper("86be7afa339d0fc7cfc785e72f578d33"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "abc", strtoupper("c14a12199c66e4ba84636b0f69144c77"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "message digest", strtoupper("9e327b3d6e523062afc1132d7df9d1b8"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "abcdefghijklmnopqrstuvwxyz", strtoupper("fd2aa607f71dc8f510714922b371834e"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", strtoupper("a1aa0689d0fafa2ddc22e88b49133a06"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper("d1e959eb179c911faea4624c60c5c702"));
    runhashtest(CRYPTOPP_HASH_RIPEMD128, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper("3f45ef194732c2dbb2c4a2c769795fa3"));
}
else
    print "Skipping RIPEMD128 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_RIPEMD160))
{
	print "RIPEMD160 tests:\n";

	runhashtest(CRYPTOPP_HASH_RIPEMD160, "", strtoupper("9c1185a5c5e9fc54612808977ee8f548b2258d31"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "a", strtoupper("0bdc9d2d256b3ee9daae347be6f4dc835a467ffe"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "abc", strtoupper("8eb208f7e05d987a9b044a8e98c6b087f15a0bfc"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "message digest", strtoupper("5d0689ef49d2fae572b881b123a85ffa21595f36"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "abcdefghijklmnopqrstuvwxyz", strtoupper("f71c27109c692c1b56bbdceb5b9d2865b3708dbc"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", strtoupper("12a053384a9c0c88e405a06c27dcf49ada62eb2b"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper("b0e20b6e3116640286ed3a87a5713079b21f5189"));
	runhashtest(CRYPTOPP_HASH_RIPEMD160, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper("9b752e45573d4b39f4dbd3323cab82bf63326bfb"));
}
else
	print "Skipping RIPEMD160 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_RIPEMD256))
{
    print "RIPEMD 256 tests:\n";

    runhashtest(CRYPTOPP_HASH_RIPEMD256, "", strtoupper("02ba4c4e5f8ecd1877fc52d64d30e37a2d9774fb1e5d026380ae0168e3c5522d"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "a", strtoupper("f9333e45d857f5d90a91bab70a1eba0cfb1be4b0783c9acfcd883a9134692925"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "abc", strtoupper("afbd6e228b9d8cbbcef5ca2d03e6dba10ac0bc7dcbe4680e1e42d2e975459b65"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "message digest", strtoupper("87e971759a1ce47a514d5c914c392c9018c7c46bc14465554afcdf54a5070c0e"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "abcdefghijklmnopqrstuvwxyz", strtoupper("649d3034751ea216776bf9a18acc81bc7896118a5197968782dd1fd97d8d5133"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", strtoupper("3843045583aac6c8c8d9128573e7a9809afb2a0f34ccc36ea9e72f16f6368e3f"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper("5740a408ac16b720b84424ae931cbb1fe363d1d0bf4017f1a89f7ea6de77a0b8"));
    runhashtest(CRYPTOPP_HASH_RIPEMD256, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper("06fdcc7a409548aaf91368c06a6275b553e3f099bf0ea4edfd6778df89a890dd"));
}
else
    print "Skipping RIPEMD256 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_RIPEMD320))
{
    print "RIPEMD320 tests\n";

    runhashtest(CRYPTOPP_HASH_RIPEMD320, "", strtoupper("22d65d5661536cdc75c1fdf5c6de7b41b9f27325ebc61e8557177d705a0ec880151c3a32a00899b8"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "a", strtoupper("ce78850638f92658a5a585097579926dda667a5716562cfcf6fbe77f63542f99b04705d6970dff5d"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "abc", strtoupper("de4c01b3054f8930a79d09ae738e92301e5a17085beffdc1b8d116713e74f82fa942d64cdbc4682d"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "message digest", strtoupper("3a8e28502ed45d422f68844f9dd316e7b98533fa3f2a91d29f84d425c88d6b4eff727df66a7c0197"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "abcdefghijklmnopqrstuvwxyz", strtoupper("cabdb1810b92470a2093aa6bce05952c28348cf43ff60841975166bb40ed234004b8824463e6b009"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", strtoupper("d034a7950cf722021ba4b84df769a5de2060e259df4c9bb4a4268c0e935bbc7470a969c9d072a1ac"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", strtoupper("ed544940c86d67f250d232c30b7b3e5770e0c60c8cb9a4cafe3b11388af9920e1b99230b843c86a4"));
    runhashtest(CRYPTOPP_HASH_RIPEMD320, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", strtoupper("557888af5f6d8ed62ab66945c6d2a0a47ecd5341e915eb8fea1d0524955f825dc717e4a008ab2d42"));
}
else
    print "Skipping RIPEMD320 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_SHA1))
{
	print "SHA1 tests\n";
	runhashtest(CRYPTOPP_HASH_SHA1, "abc", "A9993E364706816ABA3E25717850C26C9CD0D89D");
	runhashtest(CRYPTOPP_HASH_SHA1, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", "84983E441C3BD26EBAAE4AA1F95129E5E54670F1");
}
else
	print "Skipping SHA1 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_SHA256))
{
	print "SHA-256:\n";
	runhashtest(CRYPTOPP_HASH_SHA256, "abc", strtoupper("ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad"));
	runhashtest(CRYPTOPP_HASH_SHA256, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", strtoupper("248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1"));
}
else
	print "Skipping SHA-256 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_SHA384))
{
	print "SHA-384:\n";
	runhashtest(CRYPTOPP_HASH_SHA384, "abc", strtoupper("cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed8086072ba1e7cc2358baeca134c825a7"));
	runhashtest(CRYPTOPP_HASH_SHA384, "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu", strtoupper("09330c33f71147e83d192fc782cd1b4753111b173b3b05d22fa08086e3b0f712fcc7c71a557e2db966c3e9fa91746039"));
}
else
	print "Skipping SHA-384 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_SHA512))
{
	print "SHA-512:\n";
	runhashtest(CRYPTOPP_HASH_SHA512, "abc", strtoupper("ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f"));
	runhashtest(CRYPTOPP_HASH_SHA512, "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmnhijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu", strtoupper("8e959b75dae313da8cf4f72814fc143f8f7779c6eb9f7fa17299aeadb6889018501d289e4900f7e4331b99dec4b5433ac7d329eeb6dd26545e96e55b874be909"));
}
else
	print "Skipping SHA-512 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_TIGER))
{
	print "Tiger tests:\n\n";

	runhashtest(CRYPTOPP_HASH_TIGER, "", strtoupper("3293ac630c13f0245f92bbb1766e16167a4e58492dde73f3"));
	runhashtest(CRYPTOPP_HASH_TIGER, "abc", strtoupper("2aab1484e8c158f2bfb8c5ff41b57a525129131c957b5f93"));
	runhashtest(CRYPTOPP_HASH_TIGER, "Tiger", strtoupper("dd00230799f5009fec6debc838bb6a27df2b9d6f110c7937"));
	runhashtest(CRYPTOPP_HASH_TIGER, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-", strtoupper("f71c8583902afb879edfe610f82c0d4786a3a534504486b5"));
	runhashtest(CRYPTOPP_HASH_TIGER, "ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789", strtoupper("48ceeb6308b87d46e95d656112cdf18d97915f9765658957"));
	runhashtest(CRYPTOPP_HASH_TIGER, "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham", strtoupper("8a866829040a410c729ad23f5ada711603b3cdd357e4c15e"));
	runhashtest(CRYPTOPP_HASH_TIGER, "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge.", strtoupper("ce55a6afd591f5ebac547ff84f89227f9331dab0b611c889"));
	runhashtest(CRYPTOPP_HASH_TIGER, "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge, 1996.", strtoupper("631abdd103eb9a3d245b6dfd4d77b257fc7439501d1568dd"));
	runhashtest(CRYPTOPP_HASH_TIGER, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-", strtoupper("c54034e5b43eb8005848a7e0ae6aac76e4ff590ae715fd25"));
}
else
	print "Skipping Tiger tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_WHIRLPOOL))
{
    print "Whirlpool tests:\n\n";

    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "", "470F0409ABAA446E49667D4EBE12A14387CEDBD10DD17B8243CAD550A089DC0FEEA7AA40F6C2AAAB71C6EBD076E43C7CFCA0AD32567897DCB5969861049A0F5A");
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "A8583B83929BD46F0006E8401F87767FF0E23B96CD4CB2FE3774901EE6EEED91F43AB569FB908122C53A264A35687B40A0590D83E69FA82724380BAE82A1CAA0", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x40\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "E105E4B822C5C7936581F851BED55490088D1FDB20361B80052A7A04AF1E53BF2CC47479CB0EC82F88ACC2AC00B1FF336BFD16F0BBACF35F5E241895F5781676", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x20\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "681805BDB0D56D56B21425F4C6FF0991AF5A5A84B13D78BEDCE701250F470E844290C3E4EF3EB855CE6D1BA282059702F66B745837E289C865C525190078180F", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "F5F25283B2E3C73CFBC823B57D73BA6522970F12E50F9B6DEE376855855CAC52D5678AB1B1E2BE47C73CE9BDA0CEBB6ECE63723656CE298A0FE0E35281D5C9DB", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x08\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "ABB3EDC98801C5535CFA4FB450436D1C12D1A26A1F036A094EFA812F07FDC99D2B8A2386D2F1BF323E9C7184916914F8DD04D8496F9D1C9BE72A7609DA2F57AD", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "F43F0B209D2A2503A0256BE6C20E71EFF041D82896C1F4C32BA7BE7DDA69938FCD2CDF90C6526E16438D22731AC586F96582EC4E02285161D2A063F84BD33FF9", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "414AEDB80CE94767EF65C0CA724A79CC9801B9C9C9CA38A4CDB7370AA81B681C8B6F888340F0C43CF1A41364E29E9DB7C08544A0CAF5200F44A0D797038545E8", null, true);
runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "2140AC1C98D4814C5DCA77D8F0411695198B10A33C3BC8874EC2D1A792E1F80EAF278E5B5A3E6470FC9A35673194E451E31FBE7196A6313E2A53A0D6794F5BA5", null, true);
    runhashtest(CRYPTOPP_HASH_WHIRLPOOL, "\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", "7872A92AF3EA03D6CAD5DE14D96CA2922C0FDADCE175055348F875AF4A676393CF52DE9C069C51A16B03AD30D6D045CCB2216972540D9E12799D4C535F74F82A", null, true);
}
else
    print "Skipping Whirlpool tests";

line();


print "\n\nHMAC tests:\n\n";

if (cryptopp_hash_enabled(CRYPTOPP_HMAC_MD5))
{
	print "MD5-HMAC tests:\n\n";

	runhashtest(CRYPTOPP_HMAC_MD5, "Hi There", strtoupper("9294727a3638bb1c13f48ef8158bfc9d"), "0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b");
	runhashtest(CRYPTOPP_HMAC_MD5, "what do ya want for nothing?", strtoupper("750c783e6ab0b503eaa86e310a5db738"), "4A656665");
	runhashtest(CRYPTOPP_HMAC_MD5, "Test Using Larger Than Block-Size Key - Hash Key First", strtoupper("6b1ab7fe4bd7bf8f0b62e6ce61b9d0cd"), str_pad("", 160, "A"));
}
else
	print "Skipping HMAC tests";

line();


print "\n\nChecksum tests:\n\n";


if (cryptopp_hash_enabled(CRYPTOPP_HASH_ADLER32))
{
	print "Adler32 tests:\n\n";

	runhashtest(CRYPTOPP_HASH_ADLER32, "", "00000001");
	runhashtest(CRYPTOPP_HASH_ADLER32, "a", "00620062");
	runhashtest(CRYPTOPP_HASH_ADLER32, "abc", "024D0127");
	runhashtest(CRYPTOPP_HASH_ADLER32, "message digest", "29750586");
	runhashtest(CRYPTOPP_HASH_ADLER32, "abcdefghijklmnopqrstuvwxyz", "90860B20");
	runhashtest(CRYPTOPP_HASH_ADLER32, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "8ADB150C");
}
else
	print "Skipping Adler32 tests";

line();


if (cryptopp_hash_enabled(CRYPTOPP_HASH_CRC32))
{
	print "CRC32 tests:\n\n";

	runhashtest(CRYPTOPP_HASH_CRC32, "", "00000000");
	runhashtest(CRYPTOPP_HASH_CRC32, "a", "43BEB7E8");
	runhashtest(CRYPTOPP_HASH_CRC32, "abc", "C2412435");
	runhashtest(CRYPTOPP_HASH_CRC32, "message digest", "7F9D1520");
	runhashtest(CRYPTOPP_HASH_CRC32, "abcdefghijklmnopqrstuvwxyz", "BD50274C");
	runhashtest(CRYPTOPP_HASH_CRC32, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "D2E6C21F");
	runhashtest(CRYPTOPP_HASH_CRC32, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", "724AA97C");
	runhashtest(CRYPTOPP_HASH_CRC32, "123456789", "2639F4CB");
}
else
	print "Skipping CRC32 tests";

line();

$totaltests = $passedtests + $failedtests;
print "\n\n\nTests run:  $totaltests\n";
print "Passed:     $passedtests (" . ($totaltests == 0 ? 0 : round($passedtests / $totaltests * 100, 1)) . "%)\n";
print "Failed:     $failedtests (" . ($totaltests ==0 ? 0 : round($failedtests / $totaltests * 100, 1)) . "%)\n";

line();

?>
